#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez, Marta Lopez & Lauro Sumoy           ##
## Copyright (C) 2019-2021 Lauro Sumoy Lab, IGTP, Spain ##
##########################################################
'''
Calls sRNAbench to create miRNA profile
'''
## useful imports
import time
import io
import os
import re
import sys
from sys import argv
from io import open
from termcolor import colored

## import my modules
from HCGB import functions
from XICRA.config import set_config


###############
def sRNAbench_caller(reads, sample_folder, name, threads, species, Debug):
    """Passes the sample information to be analyzed by sRNAbench()

    Checks if the computation has been performed before. If not, it calls
    sRNAbench().
    
    :param reads: file with sample reads
    :param sample_folder: output folder
    :param name: sample name
    :param threads: selected threads (by defoult 2)
    :param species: species tag ID. Default: hsa (Homo sapiens)
    :param Debug: display complete log.


    :returns: True/False
    """
    # check if previously generated and succeeded
    filename_stamp = sample_folder + '/.success'
    if os.path.isfile(filename_stamp):
        stamp = functions.time_functions.read_time_stamp(filename_stamp)
        print (colored("\tA previous command generated results on: %s [%s -- %s]" %(stamp, name, 'sRNAbench'), 'yellow'))
    else:
        # Call sRNAbench
        code_returned = sRNAbench(reads, sample_folder, name, threads, species, Debug)
        if code_returned:
            functions.time_functions.print_time_stamp(filename_stamp)
        else:
            print ('** Sample %s failed...' %name)
            return(False)
        
    return(True)

###############       
def sRNAbench (reads, outpath, file_name, num_threads, species, Debug):
    """Executes the sRNAbench analyis of the sample.

    Checks if the reads are joined and builds the sRNAbench command to execute it. 
    
    :param reads: file with sample reads
    :param outpath: output folder
    :param file_name: sample name
    :param num_threads: selected threads (by defoult 2)
    :param species: species tag ID. Default: hsa (Homo sapiens)
    :param Debug: display complete log.


    :returns: The sRNAbench output
    """
    sRNAbench_exe = set_config.get_exe("sRNAbench", Debug=Debug)
    
    ## set as option
    ## sRNAbench.jar in exec/ folder within sRNAtoolboxDB
    ## sRNAbench_db = os.path.abspath(os.path.join(os.path.dirname(sRNAbench_exe), '..')) ## sRNAtoolboxDB
    
    ## sRNAbench.jar linked in bin/. sRNAtoolboxDB in same folder 
    sRNAbench_db = os.path.abspath(os.path.join(os.path.dirname(sRNAbench_exe), 'sRNAtoolboxDB')) ## sRNAtoolboxDB
    
    logfile = os.path.join(outpath, 'sRNAbench.log')
    
    if (len(reads) > 1):
        print (colored("** ERROR: Only 1 fastq file is allowed please joined reads before...", 'red'))
        exit()
    
    ## create command    
    java_exe = set_config.get_exe('java', Debug=Debug)
    cmd = '%s -jar %s dbPath=%s input=%s output=%s' %(java_exe, sRNAbench_exe, sRNAbench_db, reads[0], outpath)  
    cmd = cmd + ' microRNA=%s isoMiR=true plotLibs=true graphics=true' %species 
    cmd = cmd + ' plotMiR=true bedGraphMode=true writeGenomeDist=true'
    cmd = cmd + ' chromosomeLevel=true chrMappingByLength=true > ' + logfile 
    
    return(functions.system_call_functions.system_call(cmd))



#!/usr/bin/env python3
############################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain   ##
############################################################
"""
Create RNA biotype analysis using STAR and featureCounts
"""
## import useful modules
import os
import sys
import re
import time
from io import open
import shutil
import concurrent.futures
import pandas as pd
from termcolor import colored

## import my modules
from XICRA.config import set_config
from XICRA.modules import help_XICRA
from XICRA.scripts import RNAbiotype
from XICRA.scripts import mapReads
from XICRA.scripts import multiQC_report
from XICRA.other_tools import tools

from HCGB import sampleParser
from HCGB.functions import fasta_functions, time_functions
from HCGB.functions import aesthetics_functions, system_call_functions
from HCGB.functions import files_functions, main_functions

global mapping_results
mapping_results = {}

##############################################
def run_biotype(options):

    ## init time
    start_time_total = time.time()

    ##################################
    ### show help messages if desired    
    ##################################
    if (options.help_format):
        ## help_format option
        help_XICRA.help_fastq_format()
    elif (options.help_project):
        ## information for project
        help_XICRA.project_help()
        exit()
    elif (options.help_RNAbiotype):
        ## information for join reads
        RNAbiotype.help_info()
        exit()
        
    ## debugging messages
    global Debug
    if (options.debug):
        Debug = True
    else:
        Debug = False
        
    ### set as default paired_end mode
    if (options.single_end):
        options.pair = False
    else:
        options.pair = True
    
    aesthetics_functions.pipeline_header('XICRA')
    aesthetics_functions.boxymcboxface("RNA biotype analysis")
    print ("--------- Starting Process ---------")
    time_functions.print_time()

    ## absolute path for in & out
    input_dir = os.path.abspath(options.input)
    outdir=""

    ## set mode: project/detached
    if (options.detached):
        outdir = os.path.abspath(options.output_folder)
        options.project = False
    else:
        options.project = True
        outdir = input_dir        
    
    ## get files
    print ('+ Getting files from input folder... ')
       
    ## get files
    if options.noTrim:
        print ('+ Mode: fastq.\n+ Extension: ')
        print ("[ fastq, fq, fastq.gz, fq.gz ]\n")
        pd_samples_retrieved = sampleParser.files.get_files(options, input_dir, "fastq", ("fastq", "fq", "fastq.gz", "fq.gz"), options.debug)
        
    else:
        print ('+ Mode: trim.\n+ Extension: ')
        print ("[ _trim_ ]\n")
        pd_samples_retrieved = sampleParser.files.get_files(options, input_dir, "trim", ['_trim_'], options.debug)
        
        ## Discard if joined reads: use trimmed single-end or paired-end
        pd_samples_retrieved = pd_samples_retrieved[pd_samples_retrieved['ext'] != '_joined']   
    
    ## debug message
    if (Debug):
        print (colored("**DEBUG: pd_samples_retrieve **", 'yellow'))
        print (pd_samples_retrieved)

    ## generate output folder, if necessary
    print ("\n+ Create output folder(s):")
    if not options.project:
        files_functions.create_folder(outdir)

    ## for samples
    mapping_outdir_dict = files_functions.outdir_project(outdir, options.project, pd_samples_retrieved, "map", options.debug)
    
    ## debug message
    if (Debug):
        print (colored("**DEBUG: mapping_outdir_dict **", 'yellow'))
        print (mapping_outdir_dict)

    # time stamp
    start_time_partial = time_functions.timestamp(start_time_total)

    ## optimize threads
    name_list = set(pd_samples_retrieved["new_name"].tolist())
    threads_job = main_functions.optimize_threads(options.threads, len(name_list)) ## threads optimization
    max_workers_int = int(options.threads/threads_job)
    
    ## debug message
    if (Debug):
        print (colored("**DEBUG: options.threads " +  str(options.threads) + " **", 'yellow'))
        print (colored("**DEBUG: max_workers " +  str(max_workers_int) + " **", 'yellow'))
        print (colored("**DEBUG: cpu_here " +  str(threads_job) + " **", 'yellow'))
        
    ##############################################
    ## map Reads
    ##############################################
    start_time_partial = mapReads_module(options, pd_samples_retrieved, mapping_outdir_dict, 
                    options.debug, max_workers_int, threads_job, start_time_partial, outdir)

    ## debug message
    if (Debug):
         print (colored("**DEBUG: mapping_results **", 'yellow'))
         print (mapping_results)
    
    # time stamp
    start_time_partial = time_functions.timestamp(start_time_partial)
    
    ## for samples
    biotype_outdir_dict = files_functions.outdir_project(outdir, options.project, pd_samples_retrieved, "biotype", options.debug)
    
    ## debug message
    if (Debug):
        print (colored("**DEBUG: biotype_outdir_dict **", 'yellow'))
        print (biotype_outdir_dict)
    
    ## multimapping:
    if options.no_multiMapping:
        multimapping = False
    else:
        multimapping = True    
    
    ## get RNAbiotype information
    RNAbiotype.RNAbiotype_module_call(mapping_results, biotype_outdir_dict, options.annotation, 
                                      options.debug, max_workers_int, threads_job, multimapping, options.stranded)

    # time stamp
    start_time_partial = time_functions.timestamp(start_time_partial)
    
    if (options.skip_report):
        print ("+ No report generation...")
    else:
        print ("\n+ Generating a report using MultiQC module for featureCount analysis.")
        outdir_report = files_functions.create_subfolder("report", outdir)

        ## get subdirs generated and call multiQC report module
        givenList = []
        print ("+ Detail information for each sample could be identified in separate folders:")
        
        ## call multiQC report module
        givenList = [ v for v in biotype_outdir_dict.values() ]
        my_outdir_list = set(givenList)

        ## debug message
        if (Debug):
            print (colored("\n**DEBUG: my_outdir_list for multiqc report **", 'yellow'))
            print (my_outdir_list)
            print ("\n")
        
        featureCount_report = files_functions.create_subfolder("featureCount", outdir_report)
        multiQC_report.multiQC_module_call(my_outdir_list, "featureCount", featureCount_report,"-dd 2")
        print ('\n+ A summary HTML report of each sample is generated in folder: %s' %featureCount_report)

        ### Summarizing RNA biotype information
        biotype_report = files_functions.create_subfolder("biotype", outdir_report)
        single_files_biotype = files_functions.create_subfolder("samples", biotype_report)
        
        ## results
        dict_files = {}
        
        for samples in biotype_outdir_dict:
            featurecount_file = os.path.join(biotype_outdir_dict[samples], 'featureCount.out.tsv')
            if files_functions.is_non_zero_file(featurecount_file):
                dict_files[samples] = featurecount_file
            ## copy pdf
            pdf_plot = main_functions.retrieve_matching_files(biotype_outdir_dict[samples], '.pdf', options.debug)
            if files_functions.is_non_zero_file(pdf_plot[0]):
                shutil.copy(pdf_plot[0], single_files_biotype)
        
        ## collapse all information
        all_data = RNAbiotype.generate_matrix(dict_files)
    
        ## print into excel/csv
        print ('+ Table contains: ', len(all_data), ' entries\n')
        
        ## debugging messages
        if Debug:
            print ("** DEBUG: all_data")
            print (all_data)
        
        ## set abs_csv_outfile to be in report folder
        ## copy or link files for each sample analyzed
        abs_csv_outfile = os.path.join(biotype_report, "summary.csv")
        all_data.to_csv(abs_csv_outfile)
       
        ## create plot: call R [TODO: implement in python]
        outfile_pdf = os.path.join(biotype_report, "RNAbiotypes_summary.pdf")
        
        ## R scripts
        biotype_R_script = tools.R_scripts('plot_RNAbiotype_sum', options.debug)
        rscript = set_config.get_exe("Rscript", options.debug)
        cmd_R_plot = "%s %s -f %s -o %s" %(rscript, biotype_R_script, abs_csv_outfile, outfile_pdf)
        
        ##
        print ("+ Create summary plot for all samples")
        callCode = system_call_functions.system_call(cmd_R_plot)
            
    print ("\n*************** Finish *******************")
    start_time_partial = time_functions.timestamp(start_time_total)
    print ("\n+ Exiting join module.")
    return()

#########################################
def mapReads_module(options, pd_samples_retrieved, outdir_dict, Debug, 
                    max_workers_int, threads_job, start_time_partial, outdir):
    
    # Group dataframe by sample name
    sample_frame = pd_samples_retrieved.groupby(["new_name"])
    
    ## options
    STAR_exe = set_config.get_exe("STAR", Debug=Debug)
    cwd_folder = os.path.abspath("./")
    folder=files_functions.create_subfolder('STAR_files', cwd_folder)

    ## For many samples it will have to load genome index in memory every time.
    ## For a unique sample it will not matter. Take care genome might stay in memory.
    ## Use before loop option LoadAndExit and then:
        ## in loop
        ## Use option LoadAndKeep, set shared memory > 30 Gb
    ## when finished loop Remove memory        
    
    ## check reference
    if (options.fasta):
        print ("+ Genome fasta file provided")
        print ("+ Create genomeDir for later usage...")
        options.fasta = os.path.abspath(options.fasta)
        
        ## create genomeDir
        options.genomeDir = mapReads.create_genomeDir(folder, STAR_exe, options.threads, options.fasta, options.limitRAM)
        
    elif (options.genomeDir):
        print ("+ genomeDir provided.")
        options.genomeDir = os.path.abspath(options.genomeDir)
        
    ## remove previous reference genome from memory
    print ("+ Remove genome in memory from previous call... (if any)")
    #mapReads.remove_Genome(STAR_exe, options.genomeDir, folder, options.threads)
    
    ## load reference genome
    #mapReads.load_Genome(folder, STAR_exe, options.genomeDir, options.threads)

    ## functions.time_functions.timestamp
    start_time_partial = time_functions.timestamp(start_time_partial)
    
    print ("+ Mapping sequencing reads for each sample retrieved...")

    ## send for each sample
    with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers_int) as executor:
        commandsSent = { executor.submit(mapReads_caller, sorted(cluster["sample"].tolist()), 
                                         outdir_dict[name], name, threads_job, STAR_exe, 
                                         options.genomeDir, options.limitRAM, Debug): name for name, cluster in sample_frame }

        for cmd2 in concurrent.futures.as_completed(commandsSent):
            details = commandsSent[cmd2]
            try:
                data = cmd2.result()
            except Exception as exc:
                print ('***ERROR:')
                print (cmd2)
                print('%r generated an exception: %s' % (details, exc))

    print ("\n\n+ Mapping reads has finished...")
    
    ## functions.time_functions.timestamp
    start_time_partial = time_functions.timestamp(start_time_partial)

    ## remove reference genome from memory
    mapReads.remove_Genome(STAR_exe, options.genomeDir, folder, options.threads)
    
    ## functions.time_functions.timestamp
    start_time_partial = time_functions.timestamp(start_time_partial)

    if (options.skip_report):
        print ("+ No report generation...")
    else:
        print ("\n+ Generating a report using MultiQC module.")
        outdir_report = files_functions.create_subfolder("report", outdir)

        ## get subdirs generated and call multiQC report module
        givenList = []
        print ("+ Detail information for each sample could be identified in separate folders:")
        
        ## call multiQC report module
        givenList = [ v for v in outdir_dict.values() ]
        my_outdir_list = set(givenList)

        ## debug message
        if (Debug):
            print (colored("\n**DEBUG: my_outdir_list for multiqc report **", 'yellow'))
            print (my_outdir_list)
            print ("\n")
        
        map_report = files_functions.create_subfolder("STAR", outdir_report)
        multiQC_report.multiQC_module_call(my_outdir_list, "STAR", map_report,"-dd 2")
        print ('\n+ A summary HTML report of each sample is generated in folder: %s' %map_report)

    return(start_time_partial)

#################################
def mapReads_caller(files, folder, name, threads, STAR_exe, genomeDir, limitRAM_option, Debug):
    ## check if previously joined and succeeded
    filename_stamp = folder + '/.success'
    if os.path.isfile(filename_stamp):
        stamp = time_functions.read_time_stamp(filename_stamp)
        print (colored("\tA previous command generated results on: %s [%s -- %s]" %(stamp, name, 'STAR'), 'yellow'))
    else:
        ##
        if Debug:
            print ("\n** DEBUG: mapReads_caller options **\n")
            print ("folder: " + folder) 
            print ("name: " + name)
            print ("threads: " + str(threads))
            print ("STAR_exe: " + STAR_exe) 
            print ("genomeDir: " + genomeDir) 
            print ("limitRAM_option: " + str(limitRAM_option))
            print ("files: ")
            print (files)
            
        # Call STAR
        code_returned = mapReads.mapReads("LoadAndKeep", files, folder, name, STAR_exe, genomeDir, limitRAM_option, threads, Debug)
        
        if (code_returned):
            time_functions.print_time_stamp(filename_stamp)
        else:
            print ("+ Mapping sample %s failed..." %name)
    
    ## return results
    bam_file = os.path.join(folder, 'Aligned.sortedByCoord.out.bam')
    mapping_results[name] = bam_file

    return()
    



#from pkg_resources import get_distribution
#try:
#    __version__ = get_distribution('XICRA').version
#except:
#    __version__ = 'local'
### to include when distribution available
## version will be retrieve from setup.py


__all__ = [
	'modules',
	'scripts',
	'config',
	'other_tools'
]

from XICRA import *




#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez, Marta Lopez & Lauro Sumoy           ##
## Copyright (C) 2019-2021 Lauro Sumoy Lab, IGTP, Spain ##
##########################################################
'''
Calls mirtop to accommodate results
'''
## useful imports
import time
import io
import os
import re
import sys
from sys import argv
from io import open
from termcolor import colored

## import my modules
from HCGB import functions
from XICRA.config import set_config

###############
def miRTop_caller(results_folder, mirtop_folder, name, threads, miRNA_gff, hairpinFasta, format, species, Debug):
    """Checks if the computation has already been performed. If not, it calls
    miRTop()
    :param results_folder: file with the output of the sample from each software 
    :param folder: output miRTop folder
    :param name: sample name
    :param threads: selected threads (by defoult 2)
    :param miRNA_gff: miRNA gff3 annotation file
    :param hairpinFasta: hairpin fasta file
    :param format: 'sRNAbench', 'optimir' or 'seqbuster'
    :param species: species tag ID. Default: hsa (Homo sapiens)
    :param Debug: display complete log.
    :returns: True/False
    """
    # check if previously generated and succeeded
    mirtop_folder_gff = functions.files_functions.create_subfolder('gff', mirtop_folder)
    mirtop_folder_stats = functions.files_functions.create_subfolder('stats', mirtop_folder)
    mirtop_folder_counts = functions.files_functions.create_subfolder('counts', mirtop_folder)
    mirtop_folder_counts = functions.files_functions.create_subfolder('export', mirtop_folder)

    filename_stamp = mirtop_folder_counts + '/.success'
    if os.path.isfile(filename_stamp):
        stamp = functions.time_functions.read_time_stamp(filename_stamp)
        print (colored("\tA previous command generated results on: %s [%s -- %s]" %(stamp, name, 'miRTop'), 'yellow'))
    else:
        # Call miRTop
        code_returned = miRTop(results_folder, mirtop_folder, name, threads, format.lower(), miRNA_gff, hairpinFasta, species, Debug)
        if code_returned:
            functions.time_functions.print_time_stamp(filename_stamp)
        else:
            print ('** Sample %s failed...' %name)
            return(False)
        
        return(True)

###############
def miRTop(results_folder, sample_folder, name, threads, format, miRNA_gff, hairpinFasta, species,Debug):
    """Checks if the computation has already been performed. If not, it calls
    miRTop()

    :param results_folder: file with the output of the sample from each software 
    :param folder: output miRTop folder
    :param name: sample name
    :param threads: selected threads (by defoult 2)
    :param miRNA_gff: miRNA gff3 annotation file
    :param hairpinFasta: hairpin fasta file
    :param format: 'sRNAbench', 'optimir' or 'seqbuster'
    :param species: species tag ID. Default: hsa (Homo sapiens)
    :param Debug: display complete log.


    :returns: Folder with the results for the sample in miRTop format
    """

    miRTop_exe = set_config.get_exe('miRTop', Debug=Debug)
    logfile = os.path.join(sample_folder, name + '.log')
    
    ## folders
    mirtop_folder_gff = os.path.join(sample_folder, 'gff')
    mirtop_folder_stats = os.path.join(sample_folder, 'stats')
    mirtop_folder_counts = os.path.join(sample_folder, 'counts')
    mirtop_folder_export = os.path.join(sample_folder, 'export')
    
    ## get info according to software
    if format == "sRNAbench":
        ## get sRNAbench info
        reads_annot = os.path.join(results_folder, "reads.annotation")
        
        ## check non zero
        if not functions.files_functions.is_non_zero_file(reads_annot):
            print (colored("\tNo isomiRs detected for sample [%s -- %s]" %(name, 'sRNAbench'), 'yellow'))
            return (False)
            
    elif format == "optimir":
        ## get optimir info
        gff3_file = functions.main_functions.retrieve_matching_files(os.path.join(results_folder, "OptimiR_Results"), "gff3", Debug)[0]
        results_folder = gff3_file
        
        ## check non zero
        if not functions.files_functions.is_non_zero_file(gff3_file):
            print (colored("\tNo isomiRs detected for sample [%s -- %s]" %(name, 'optimir'), 'yellow'))
            return (False)
    
    elif format == "seqbuster":
        ## get miraligner info
        mirna_file = functions.main_functions.retrieve_matching_files(results_folder, ".mirna", Debug)[0]
        results_folder = mirna_file
        
        ## check non zero
        if not functions.files_functions.is_non_zero_file(mirna_file):
            print (colored("\tNo isomiRs detected for sample [%s -- %s]" %(name, 'miraligner'), 'yellow'))
            return (False)
    
        
    ## miRTop analysis gff
    filename_stamp_gff = mirtop_folder_gff + '/.success'
    if os.path.isfile(filename_stamp_gff):
        stamp = functions.time_functions.read_time_stamp(filename_stamp_gff)
        print (colored("\tA previous command generated results on: %s [%s -- %s - gff]" %(stamp, name, 'miRTop'), 'yellow'))
    else:
        print ('Creating isomiRs gtf file for sample %s' %name)
        cmd = miRTop_exe + ' gff --sps %s --hairpin %s --gtf %s --format %s -o %s %s 2> %s' %(
                                                    species, hairpinFasta, miRNA_gff, format,
                                                    mirtop_folder_gff, results_folder, logfile)
        
        ## execute
        code_miRTop = functions.system_call_functions.system_call(cmd)
        if code_miRTop:
            functions.time_functions.print_time_stamp(filename_stamp_gff)
        else:
            return(False)
        
    ## miRTop stats
    mirtop_folder_gff_file = os.path.join(mirtop_folder_gff, 'mirtop.gff')

    #filename_stamp_stats = mirtop_folder_stats + '/.success'
    #if os.path.isfile(filename_stamp_stats):
    #    stamp = functions.time_functions.read_time_stamp(filename_stamp_stats)
    #    print (colored("\tA previous command generated results on: %s [%s -- %s - stats]" %(stamp, name, 'miRTop'), 'yellow'))
    #else:
    #    print ('Creating isomiRs stats for sample %s' %name)
    #    cmd_stats = miRTop_exe + ' stats -o %s %s 2>> %s' %(mirtop_folder_stats, mirtop_folder_gff_file, logfile)
    #    code_miRTop_stats = functions.system_call_functions.system_call(cmd_stats)
    #    if code_miRTop_stats:
    #        functions.time_functions.print_time_stamp(filename_stamp_stats)
    #    else:
    #        return(False)
            
    ## miRTop counts
    filename_stamp_counts = mirtop_folder_counts + '/.success'
    if os.path.isfile(filename_stamp_counts):
        stamp = functions.time_functions.read_time_stamp(filename_stamp_counts)
        print (colored("\tA previous command generated results on: %s [%s -- %s - counts]" %(stamp, name, 'miRTop'), 'yellow'))
    else:
        print ('Creating isomiRs counts for sample %s' %name)
        ## if both succeeded
        cmd_stats = miRTop_exe + ' counts -o %s --gff %s --hairpin %s --gtf %s --sps %s 2>> %s' %(mirtop_folder_counts, mirtop_folder_gff_file, hairpinFasta, miRNA_gff, species, logfile)
        code_miRTop_counts = functions.system_call_functions.system_call(cmd_stats)
        
        if code_miRTop_counts:
            functions.time_functions.print_time_stamp(filename_stamp_counts)
        else:
            return(False)
    
    ## miRTop export
    filename_stamp_export = mirtop_folder_export + '/.success'
    if os.path.isfile(filename_stamp_export):
        stamp = functions.time_functions.read_time_stamp(filename_stamp_export)
        print (colored("\tA previous command generated results on: %s [%s -- %s - export]" %(stamp, name, 'miRTop'), 'yellow'))
    else:
        print ('Creating isomiRs export information for sample %s' %name)
        ## if both succeeded
        cmd_export = miRTop_exe + ' export -o %s --hairpin %s --gtf %s --sps %s --format isomir %s 2> %s' %(mirtop_folder_export, hairpinFasta, miRNA_gff, species, mirtop_folder_gff_file, logfile)
        code_miRTop_export = functions.system_call_functions.system_call(cmd_export)
        
        if code_miRTop_export:
            functions.time_functions.print_time_stamp(filename_stamp_export)
        else:
            return(False)
    
    ## return all success
    outdir_tsv = os.path.join(mirtop_folder_counts, 'mirtop.tsv')
    return (outdir_tsv)
    
    ## if any command failed
    #return(False)
    



#!/usr/bin/env python3
##########################################################
## this modules is an idea from ARIBA					##
## (https://github.com/sanger-pathogens/ariba)			##
## give credit to them appropiately						##
##														##
## Jose F. Sanchez										##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain	##
##########################################################
"""
Provides external programs details and configuration.
"""

## useful imports
import os
import io
import sys
import re
import shutil
from io import open
from sys import argv
import subprocess
import pandas as pd
from termcolor import colored
from distutils.version import LooseVersion
import pkg_resources

## import my modules
import HCGB.functions.main_functions as HCGB_main
from XICRA.config import set_config

####################################################################
def file_list(wanted_data):
	"""
	Retrieves information of additional files under folder ``XICRA/config``.

	Using :func:`HCGB.functions.main_functions.get_fullpath_list` retrieves absolute
	path for file of interest.

	:param wanted_data: name for file
	:type wanted_data: string

	:returns: Absolute path for file wanted

	"""
	config_folder = os.path.dirname(os.path.realpath(__file__))
	listOffiles = HCGB_main.get_fullpath_list(config_folder, False)
	for f in listOffiles:
		name = os.path.splitext(os.path.basename(f))[0]
		if (name == wanted_data):
			return (f)


##################
## Software
##################
def read_dependencies():
	"""Returns a dictionary containing the executable name for each software.

	It uses :func:`XICRA.config.extern_progs.file_list` to retrieve absolute path
	for file :file:`XICRA/config/software/dependencies.csv`. It then reads csv into pandas
	dataframe using :func:`XICRA.scripts.functions.main_functions.get_data` and returns it.	
	"""

	## read from file: prog2default.csv
	dependencies_file = file_list("dependencies")
	return(HCGB_main.get_data(dependencies_file, ',', 'index_col=0'))

#######################
def return_defatult_soft(soft):
	"""Returns default name for a given software name

	For some software we provide a shorter name for the software. Here, we read file
	:file:`XICRA/config/software/dependencies.csv` using :func:`XICRA.config.extern_progs.read_dependencies`
	and retrieve original software name.

	:param soft: Software name
	:type soft: string

	:returns: String with original software name

	.. seealso:: This function depends on other XICRA functions:

		- :func:`XICRA.config.extern_progs.read_dependencies`

	"""
	dependencies_df = read_dependencies()
	return(dependencies_df.loc[soft,"soft_name"])

##################
def return_min_version_soft(soft):
	"""Retrieve version for a given software

	Retrieves minimum version for the software of interest stored in :file:`XICRA/config/software/dependencies.csv`.
	It reads file using :func:`XICRA.config.extern_progs.read_dependencies`
	and retrieve minimum version required.

	:param soft: Software name
	:type soft: string

	:returns: String with minimum version

	.. seealso:: This function depends on other XICRA functions:

		- :func:`XICRA.config.extern_progs.read_dependencies`	
	"""
	dependencies_df = read_dependencies()
	return(dependencies_df.loc[soft,"min_version"])
##################

##################
def print_dependencies():
	"""

	"""
	progs = {}
	depencencies_pd = read_dependencies()
	for prog in depencencies_pd:
		#print (prog)
		prog_exe = set_config.get_exe(prog)
		#print (prog + '\t' + prog_exe)
		prog_ver = get_version(prog, prog_exe)
		progs[prog] = [prog_exe, prog_ver]

	df_programs = pd.DataFrame.from_dict(progs, orient='index', columns=('Executable path', 'Version'))
	df_programs = df_programs.stack().str.lstrip().unstack()
	pd.set_option('display.max_colwidth', -1)
	pd.set_option('display.max_columns', None)
	print (df_programs)

##################
### Python packages
##################
def min_python_module_version():
	"""Returns a dictionary containing minimum version for each python package.

	Reads information from :file:`XICRA/config/python/python_requirements.csv`.

	:returns: dictionary
	"""
	## read from file: prog2default.csv
	python_modules = file_list("python_requirements")
	package_min_versions = HCGB_main.file2dictionary(python_modules, ",")

	return(package_min_versions)
##################

##################
def return_min_version_python_package(package):
	"""
	Retrieves minimum version requirement for the given package.

	It retrieves the requirements using :func:`XICRA.config.extern_progs.min_python_module_version`
	and returns the given package requested minimun version.

	:param package:  
	:type package: string	
	:returns: Minimum version package (string)

	"""
	version_package = min_python_module_version()
	return (version_package[package])

##################
def print_package_version():
	"""
	Prints the package version required by ``XICRA``

	It retrieves the requirements using :func:`XICRA.config.extern_progs.min_python_module_version`
	and prints them using function :func:`XICRA.config.set_config.print_module_comparison`.

	:returns: Print messages
	"""
	my_packages = min_python_module_version()
	for each in my_packages:
		set_config.print_module_comparison(each, my_packages[each], 'green')



#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez                                      ##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain      ##
##########################################################
'''
Calls FASTQC analysis and parses results generated.
'''
## useful imports
import time
import io
import os
import re
import sys
from sys import argv
from io import open
from termcolor import colored

## import my modules
from HCGB import functions
from XICRA.config import set_config

############
def call_fastqc(path, files, sample, fastqc_bin, threads):    
    ## call system for fastqc sample given
    
    #name = functions.files_functions.create_subfolder(sample, path)
    logFile = path + '/' + sample + '.log'
    
    # create string with files path
    files_string = " ".join(files)
    
    ##print ("+ Calling fastqc for samples...")    
    cmd_fastqc = '%s --extract -t %s -o %s %s > %s 2> %s' %(fastqc_bin, threads, path, files_string, logFile, logFile)
    fastq_code = functions.system_call_functions.system_call( cmd_fastqc )
    
    if not fastq_code:
        print ('** Sample %s failed...' %sample)

    ## send command    
    return (fastq_code)
        
############
def run_module_fastqc(path, files, sample, threads):    
    ## Arguments provided via ARGVs

    ## check if previously done and succeeded
    filename_stamp = path + '/.success'
    if os.path.isfile(filename_stamp):
        stamp = functions.time_functions.read_time_stamp(filename_stamp)
        print (colored("\tA previous command generated results on: %s [%s -- %s]" %(stamp, sample, 'fastqc'), 'yellow'))
    else:
        ## call fastqc
        fastqc_bin = set_config.get_exe('fastqc')
        codeReturn = call_fastqc(path, files, sample, fastqc_bin, threads)

        if codeReturn:
            functions.time_functions.print_time_stamp(filename_stamp)
        
        return ()



#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez                                      ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain ##
##########################################################

## useful imports
import time
import io
import os
import sys
from termcolor import colored
from distutils.version import LooseVersion

## import my modules
import HCGB.functions.time_functions as HCGB_time
import HCGB.functions.aesthetics_functions as HCGB_aes
from XICRA.config import extern_progs
from XICRA.config import set_config

################
def run_config(options):
    ## init time
    start_time_total = time.time()

    ## debugging messages
    global Debug
    if (options.debug):
        Debug = True
    else:
        Debug = False

    HCGB_aes.pipeline_header('XICRA')
    HCGB_aes.boxymcboxface("Pipeline Configuration")
    print ("--------- Starting Process ---------")
    HCGB_time.print_time()

    print ("\nCheck dependencies, modules or third party software and print report...")

    ## python version
    HCGB_aes.print_sepLine("+", 20, False)
    print ('Python:')
    HCGB_aes.print_sepLine("+", 20, False)

    this_python_version = str(sys.version)
    python_min_version = extern_progs.return_min_version_soft('python')
    if LooseVersion(this_python_version) >= LooseVersion(python_min_version):
        print (colored("Minimum version (%s) satisfied: %s" %( python_min_version, this_python_version), 'green'))
    else:
        print (colored("Minimum version (%s) not satisfied: %s" %(python_min_version, this_python_version), 'red'))
        exit()
        
    ## third-party software
    print ('\n')
    HCGB_aes.print_sepLine("+", 20, False)
    print ('External dependencies:')
    HCGB_aes.print_sepLine("+", 20, False)
    
    set_config.check_dependencies(Debug)
    print ('\n')    

    ## python packages
    print ('\n')
    HCGB_aes.print_sepLine("+", 20, False)
    print ('Python packages:')
    HCGB_aes.print_sepLine("+", 20, False)

    set_config.check_python_packages(Debug)
    HCGB_aes.print_sepLine("+", 20, False)
    print ('\n')


    ## R packages
    print ('\n')
    HCGB_aes.print_sepLine("+", 20, False)
    print ('R packages:')
    HCGB_aes.print_sepLine("+", 20, False)

    set_config.check_R_packages(Debug)
    HCGB_aes.print_sepLine("+", 20, False)
    print ('\n')

    


#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez                                      ##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain      ##
##########################################################
"""
Help messages for different scripts, modules
"""
from termcolor import colored
from HCGB import functions

###############
def help_fastq_format():
    """
    Explanation of fastq format details.
    """

    functions.aesthetics_functions.boxymcboxface("Name format for samples")

    print ("Format for fastq files can be:")
    print ("name.fastq.gz")
    print ("name_1.fastq.gz, adding '1' or '2' to specify the read")
    print ("name_R2.fastq.gz, adding 'R1' or 'R2' to specify the read")
    print ("name_L001_R1.fastq.gz, adding the lane information as L00x after the name")
    print ("name_L001_R1_001.fastq.gz, adding 00X at the end. This naming is useful when the fastq")
    print ("files of the sample sample had been cut in different files).")
    print ("name_L001_XYZ_R1_001.fastq.gz, there can be extra info for each file.")
    print ("\nThere are many options and here we provide some guidelines on the name format.")
    print ("\n")

    functions.aesthetics_functions.print_sepLine("*",20,"red")
    print ("[1] Length limitation")
    functions.aesthetics_functions.print_sepLine("*",20,"red")
    print ("There is a limitation for the sample ID ('name') of 25 characters.")
    print (colored("** XICRA provides an option to rename samples if necessary: module prep option --rename **", 'yellow'))
    print ("\n")

    functions.aesthetics_functions.print_sepLine("*",20,"red")
    print ("[2] Single end files")
    functions.aesthetics_functions.print_sepLine("*",20,"red")
    print (colored('** Use option --single-end in the different XICRA modules **', 'yellow'))
    print ("name.fastq.gz")
    print ("name.fastq")
    print ("name.fq")
    print ("\n")

    functions.aesthetics_functions.print_sepLine("*",20,"red")
    print ("[3] Paired-end files")
    functions.aesthetics_functions.print_sepLine("*",20,"red")
    print ("Paired-end files are full supported. The format for these files are:")
    print ("Read1 => name_1.fastq.g or name_R1.fastq.gz")
    print ("Read2 => name_2.fastq.gz or name_R2.fastq.gz")
    print (colored('** See additional details for Lane information **', 'yellow'))
    print ("\n")
    
    functions.aesthetics_functions.print_sepLine("*",55,"red")
    print ("[4] Sample identification:")
    functions.aesthetics_functions.print_sepLine("*",55,"red")
    
    print ("XICRA will store the names of all the input files. After that, it will identify the samples.")
    print ("It can be the case that more than one file belong to the same sample. In order to pass this information")
    print ("to XICRA a combination of the following parameters may be needed:")
    print ("\n")

    functions.aesthetics_functions.print_sepLine("*",55,"red")
    print ("[4.1] Lane information:")
    functions.aesthetics_functions.print_sepLine("*",55,"red")
    print ("In some cases, files might contain lane information (*L00x* and/or *00x*).")
    print ("XICRA supports these names as long as follow these examples:")
    print ("name_L00x_R1.fastq.gz\tname_L00x_R2.fastq.gz")
    print ("name_L00x_1.fastq.gz\tname_L00x_2.fastq.gz")
    #print ("name_L00x_R1.fastq.gz\tname_L00x_R2.fastq.gz")
    print ("name_L00x_R1_00x.fastq.gz\tname_L00x_R2_00x.fastq.gz")
    print ("\n")
    
    print ("- If you want to include lane tags (*L00X*) into each  each sample name (differentiate samples considering the lane):")
    print (colored("** Use option --include-lane within each module and the lane tag will also be used to identify samples", 'yellow'))
    
    print ("\n- However, if you want to consider as a single sample the different lanes, you need to merge")
    print ("the fastq files from the different lanes, use option --merge_Reads within module prep.")
    print("As an example:")
    print (colored("** Options --merge_Reads within module prep **", 'yellow'))
    print ("sample1_L001_R1.fastq.gz\tsample1_L001_R2.fastq.gz")
    print ("sample1_L002_R1.fastq.gz\tsample1_L002_R2.fastq.gz")
    print ("sample1_L003_R1.fastq.gz\tsample1_L003_R2.fastq.gz")
    print ("sample1_L004_R1.fastq.gz\tsample1_L004_R2.fastq.gz")
    print ("Result:")
    print ("--------------------------------------------------")
    print ("sample1_R1.fastq.gz\tsample1_R2.fastq.gz")
    print ("\n")
    
    print ("\n- If you need to merge fastq files of the same lane that differ in the last group of numbers")
    print ("use option --mergeReads together with --include-lane within module prep.")
    print (colored("\n** Option --include_lane --merge-by-lane within module prep **", 'yellow'))
    print ("sample1_L001_R1_001.fastq.gz\tsample1_L001_R2_001.fastq.gz")
    print ("sample1_L001_R1_002.fastq.gz\tsample1_L001_R2_002.fastq.gz")
    print ("sample1_L002_R1_001.fastq.gz\tsample1_L002_R2_001.fastq.gz")
    print ("sample1_L002_R1_002.fastq.gz\tsample1_L002_R2_002.fastq.gz")
    print ("--------------------------------------------------")
    print ("Result:")
    print ("sample1_L001_R1.fastq.gz\tsample1_L001_R2.fastq.gz")
    print ("sample1_L002_R1.fastq.gz\tsample1_L002_R2.fastq.gz")
    print (colored("** Remember to use option --include_lane within each module", 'yellow'))
    print ("\n")
    
    ### if you want to merge lane and extension --mergeReads
    print ("- If you need to merge fastq files with different lanes and final extension ")
    print ("(_001, _002, ...), use only option --merge_Reads within module prep.")
    print("As an example:")
    print (colored("\n** Options --merge_Reads within module prep **", 'yellow'))
    print ("sample1_L001_R1_001.fastq.gz\tsample1_L001_R2_001.fastq.gz")
    print ("sample1_L001_R1_002.fastq.gz\tsample1_L001_R2_002.fastq.gz")
    print ("sample1_L002_R1_001.fastq.gz\tsample1_L002_R2_001.fastq.gz")
    print ("sample1_L002_R1_002.fastq.gz\tsample1_L002_R2_002.fastq.gz")
    print ("--------------------------------------------------")
    print ("Result:")
    print ("sample1_R1.fastq.gz\tsample1_R2.fastq.gz")
    print ("\n")    
    
    
    functions.aesthetics_functions.print_sepLine("*",55,"red")
    print ("[4.2] Include all information:")
    functions.aesthetics_functions.print_sepLine("*",55,"red")
    print ("In some cases, files might contain other extra information and it is necessary to ")
    print ("include it all as a tag name, in that case use --include-all. In the following example")
    print ("XYZ is the extra information and it is also used to identify each sample:")
    print ("sample1_L001_XYZ_R1_001.fastq.gz\tsample1_L001_XYZ_R2_001.fastq.gz")
    print (colored("** Remember to use option --include_all within each module", 'yellow'))
    
    print (colored("** It might be appropriate to change samples names using --rename option under prep module", 'yellow'))
    
    print ("\n- If you need to merge fastq files that only differ in the last group of numbers ")
    print ("(_001, _002, ...), use option --merge_Reads within module prep together with --include-all.")
    print("As an example:")
    print (colored("\n** Options --include_all --merge_Reads within module prep **", 'yellow'))
    print ("sample1_L001_XYZ_R1_001.fastq.gz\tsample1_L001_XYZ_R2_001.fastq.gz")
    print ("sample1_L001_XYZ_R1_002.fastq.gz\tsample1_L001_XYZ_R2_002.fastq.gz")
    print ("sample1_L002_XYZ_R1_001.fastq.gz\tsample1_L002_XYZ_R2_001.fastq.gz")
    print ("sample1_L002_XYZ_R1_002.fastq.gz\tsample1_L002_XYZ_R2_002.fastq.gz")
    print ("--------------------------------------------------")
    print ("Result:")
    print ("sample1_L001_XYZ_R1.fastq.gz\tsample1_L001_XYZ_R2.fastq.gz")
    print ("sample1_L002_XYZ_R1.fastq.gz\tsample1_L002_XYZ_R2.fastq.gz")
    print (colored("** Remember to use option --include_all within each module", 'yellow'))
    print ("\n")
    
    print ("\n")
    functions.aesthetics_functions.print_sepLine("*",15,"red")
    print ("[4.3] Extensions:")
    functions.aesthetics_functions.print_sepLine("*",15,"red")
    print ("name_L00x_R2.fastq\tname_L00x_R2.fq\nname_L00x_R2.fastq.gz\tname_L00x_R2.fq.gz")
    print ("\n")

###############
def project_help():
    return ()

###############
def multiqc_help():
    return ()

###############
def print_help_adapters():
    return()

###############
def help_join_reads():
    return ()

###############
def help_miRNA():
    return ()

###############
def help_tRNA():
    return ()




#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain        ##
##########################################################
"""
Joins paired-end sequence reads that overlap.
"""
## import useful modules
import os
import sys
import re
import time
from io import open
import shutil
import concurrent.futures
from termcolor import colored

## import my modules
from XICRA.modules import help_XICRA
from XICRA.config import set_config
from HCGB import functions
from HCGB import sampleParser

##############################################
def run_join(options):

    ## init time
    start_time_total = time.time()

    ##################################
    ### show help messages if desired    
    ##################################
    if (options.help_format):
        ## help_format option
        help_XICRA.help_fastq_format()
    elif (options.help_project):
        ## information for project
        help_XICRA.project_help()
        exit()
    elif (options.help_join_reads):
        ## information for join reads
        help_XICRA.help_join_reads()
        exit()
        
    ## debugging messages
    global Debug
    if (options.debug):
        Debug = True
    else:
        Debug = False
        
    ### set as default paired_end mode
    if (options.single_end):
        options.pair = False
    else:
        options.pair = True
    
    functions.aesthetics_functions.pipeline_header('XICRA')
    functions.aesthetics_functions.boxymcboxface("Join paired-end reads")
    print ("--------- Starting Process ---------")
    functions.time_functions.print_time()

    ## absolute path for in & out
    input_dir = os.path.abspath(options.input)
    outdir=""

    ## set mode: project/detached
    if (options.detached):
        outdir = os.path.abspath(options.output_folder)
        options.project = False
    else:
        options.project = True
        outdir = input_dir        
    
    ## Percentage difference for joining sequences
    if not options.perc_diff:
        options.perc_diff = 0
    
    print ('+ Getting files from input folder... ')
    ## get files
    if options.noTrim:
        print ('+ Mode: fastq.\n+ Extension: ')
        print ("[ fastq, fq, fastq.gz, fq.gz ]\n")
        pd_samples_retrieved = sampleParser.files.get_files(options, input_dir, "fastq", ("fastq", "fq", "fastq.gz", "fq.gz"), options.debug)
    else:
        print ('+ Mode: trim.\n+ Extension: ')
        print ("[ _trim_ ]\n")
        pd_samples_retrieved = sampleParser.files.get_files(options, input_dir, "trim", ['_trim_'], options.debug)
    
    ## debug message
    if (Debug):
        print (colored("**DEBUG: pd_samples_retrieve **", 'yellow'))
        print (pd_samples_retrieved)

    ## generate output folder, if necessary
    print ("\n+ Create output folder(s):")
    if not options.project:
        functions.files_functions.create_folder(outdir)
    ## for samples
    outdir_dict = functions.files_functions.outdir_project(outdir, options.project, pd_samples_retrieved, "join", options.debug)
    
    ## optimize threads
    name_list = set(pd_samples_retrieved["new_name"].tolist())
    threads_job = functions.main_functions.optimize_threads(options.threads, len(name_list)) ## threads optimization
    max_workers_int = int(options.threads/threads_job)

    ## debug message
    if (Debug):
        print (colored("**DEBUG: options.threads " +  str(options.threads) + " **", 'yellow'))
        print (colored("**DEBUG: max_workers " +  str(max_workers_int) + " **", 'yellow'))
        print (colored("**DEBUG: cpu_here " +  str(threads_job) + " **", 'yellow'))

    print ("+ Joining paired-end sequencing reads for each sample retrieved...")    
    
    # Group dataframe by sample name
    sample_frame = pd_samples_retrieved.groupby(["new_name"])
    
    ## send for each sample
    with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers_int) as executor:
        commandsSent = { executor.submit(fastqjoin_caller, sorted(cluster["sample"].tolist()), 
                                         outdir_dict[name], name, threads_job, options.perc_diff,
                                         Debug): name for name, cluster in sample_frame }

        for cmd2 in concurrent.futures.as_completed(commandsSent):
            details = commandsSent[cmd2]
            try:
                data = cmd2.result()
            except Exception as exc:
                print ('***ERROR:')
                print (cmd2)
                print('%r generated an exception: %s' % (details, exc))

    print ("\n\n+ Joining reads has finished...")
    
    ## TODO: create statistics on joined reads
    ##

    print ("\n*************** Finish *******************")
    start_time_partial = functions.time_functions.timestamp(start_time_total)
    print ("\n+ Exiting join module.")
    return()

#############################################
def fastqjoin_caller(list_reads, sample_folder, name, threads, perc_diff, Debug):
    ## check if previously joined and succeeded
    filename_stamp = sample_folder + '/.success'
    if os.path.isfile(filename_stamp):
        stamp = functions.time_functions.read_time_stamp(filename_stamp)
        print (colored("\tA previous command generated results on: %s [%s -- %s]" %(stamp, name, 'fastqjoin'), 'yellow'))
    else:
        # Call fastqjoin
        fastqjoin_exe = set_config.get_exe('fastqjoin')
        code_returned = fastqjoin(fastqjoin_exe, list_reads, sample_folder, name, threads, perc_diff, Debug)
        if code_returned:
            functions.time_functions.print_time_stamp(filename_stamp)
        else:
            print ('** Sample %s failed...' %name)

#############################################
def fastqjoin (fastqjoin_exe, reads, path, sample_name, num_threads, perc_diff, Debug):
    """
    
    :param fastqjoin_exe:
    :param reads:
    :param path:
    :param sample_name:
    :param num_threads: 
    :param Debug:
    
    :type fastqjoin_exe:
    :type reads:
    :type path:
    :type sample_name:
    :type num_threads: 
    :type Debug:
    
    """
    logfile = os.path.join(path, sample_name + '.fastqjoin.log')
    joined_reads = os.path.join(path, sample_name + '_trim_joined.fastq')
    unjoined_1 = os.path.join(path, sample_name + '_trimmed_unjoin_R1.fastq')
    unjoined_2 = os.path.join(path, sample_name + '_trimmed_unjoin_R2.fastq')
    
    ## check paired-end file
    if (len(reads) == 2):
        cmd = fastqjoin_exe + ' -p %s %s %s -o %s -o %s -o %s > %s' %(perc_diff, reads[0], 
                                                                  reads[1], unjoined_1, unjoined_2, 
                                                                  joined_reads, logfile)
    else:
        print ('** Wrong number of files provided for sample: %s...' %sample_name)
        return(False)

    return(functions.system_call_functions.system_call(cmd))
    


#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez, Marta Lopez & Lauro Sumoy           ##
## Copyright (C) 2019-2021 Lauro Sumoy Lab, IGTP, Spain ##
##########################################################
'''
Calls cutadapt to trim raw reads
'''
## useful imports
import time
import io
import os
import re
import sys
from sys import argv
from io import open
from termcolor import colored

## import my modules
from HCGB import functions
from XICRA.config import set_config

#############################################
def caller(list_reads, sample_folder, name, threads, min_read_len, Debug, adapters, extra):
    ## check if previously trimmed and succeeded
    filename_stamp = sample_folder + '/.success'
    if os.path.isfile(filename_stamp):
        stamp = functions.time_functions.read_time_stamp(filename_stamp)
        print (colored("\tA previous command generated results on: %s [%s -- %s]" %(stamp, name, 'cutadapt'), 'yellow'))
    else:
        # Call cutadapt
        cutadapt_exe = set_config.get_exe('cutadapt')
        code_returned = cutadapt(cutadapt_exe, list_reads, sample_folder, name, threads, min_read_len, Debug, adapters, extra)
        if code_returned:
            functions.time_functions.print_time_stamp(filename_stamp)
        else:
            print ('** Sample %s failed...' %name)


#############################################
def cutadapt(cutadapt_exe, reads, path, sample_name, num_threads, min_len_given, Debug, adapters, extra):
    """
    
    :param cutadapt_exe:
    :param reads:
    :param path:
    :param sample_name:
    :param num_threads: 
    :param Debug:
    :param adapters
    :param extra:
    
    :type cutadapt_exe:
    :type reads:
    :type path:
    :type sample_name:
    :type num_threads: 
    :type Debug:
    :type adapters: dictionary
    :type extra: string
    
    """
    logfile = os.path.join(path, sample_name + '.cutadapt.log')
    
    if (len(reads) == 2):
        if not adapters['adapter_a'] or not adapters['adapter_A']:
             print ("** ERROR: Missing adapter information")
             exit()
        
        if extra:
            o_param = os.path.join(path, sample_name + '_temp1_trim_R1.fastq')
            p_param = os.path.join(path, sample_name + '_temp1_trim_R2.fastq')
        else:
            p_param = os.path.join(path, sample_name + '_trim_R2.fastq')
            o_param = os.path.join(path, sample_name + '_trim_R1.fastq')
        
        ## paired-end mode, 15 bps as the min length cutoff
        cmd = '%s -j %s -m %s -a %s -A %s -o %s -p %s %s %s > %s' %(cutadapt_exe,  
                                                                       num_threads, min_len_given, 
                                                                       adapters['adapter_a'], 
                                                                       adapters['adapter_A'], o_param, 
                                                                       p_param, reads[0], reads[1], logfile)
    elif (len(reads) == 1):
        if not adapters['adapter_a']:
             print ("** ERROR: Missing adapter information")
             exit()

        if extra:
            o_param = os.path.join(path, sample_name + '_temp1_trim.fastq')
        else:
            o_param = os.path.join(path, sample_name + '_trim.fastq')
        
        ## single-end mode:
        cmd = '%s -j %s -m %s -a %s -o %s %s > %s' %(cutadapt_exe, num_threads, 
                                                     min_len_given,
                                                     adapters['adapter_a'], 
                                                     o_param, reads[0], logfile)    
    else:
        print ('** Wrong number of files provided for sample: %s...' %sample_name)
        return(False)

    ##
    code = functions.system_call_functions.system_call(cmd)

    ## if additional options, run a second cutadapt command
    ## to ensure this options take effect.
    if (extra):
        if (len(reads) == 2):
            o_param2 = os.path.join(path, sample_name + '_trim_R1.fastq')
            p_param2 = os.path.join(path, sample_name + '_trim_R2.fastq')
        
            ## paired-end mode
            extra_cmd = '%s %s -j %s -m %s -a %s -A %s -o %s -p %s %s %s >> %s' %(cutadapt_exe, 
                                                                                  extra, 
                                                                                  num_threads, 
                                                                                  min_len_given, 
                                                                                  adapters['adapter_a'], 
                                                                                  adapters['adapter_A'],
                                                                                  o_param2, p_param2, 
                                                                                  o_param, p_param, logfile)
        
        elif (len(reads) == 1):
            o_param2 = os.path.join(path, sample_name + '_trim.fastq')
            ## single-end mode:
            extra_cmd = '%s %s -j %s -m %s -a %s -o %s %s >> %s' %(cutadapt_exe, 
                                                                   extra, 
                                                                   num_threads, 
                                                                   min_len_given,
                                                                   adapters['adapter_a'], 
                                                                   o_param2, o_param, logfile)    
        
        code2 = functions.system_call_functions.system_call(extra_cmd)
        
        ## remove: o_param p_param
        if (len(reads) == 2):        
            os.remove(p_param)
       
        os.remove(o_param)
        return (code2)
    
    else:
        return (code)



#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez, Marta Lopez & Lauro Sumoy           ##
## Copyright (C) 2019-2021 Lauro Sumoy Lab, IGTP, Spain ##
##########################################################
'''
Calls MINTMap to create tRNA profile
'''
## useful imports
import time
import io
import os
import re
import sys
from sys import argv
from io import open
from termcolor import colored

## import my modules
from HCGB import functions
from XICRA.config import set_config
import HCGB.functions.aesthetics_functions as HCGB_aes

############################
def MINTmap_caller(MINTmap_folder, reads, name, num_threads, species, Debug):
    # check if previously generated and succeeded
    filename_stamp = MINTmap_folder + '/.success_all'
    if os.path.isfile(filename_stamp):
        stamp = functions.time_functions.read_time_stamp(filename_stamp)
        print (colored("\tA previous command generated results on: %s [%s -- %s]" %(stamp, name, 'MINTmap'), 'yellow'))
        return True

    else:
        # Call MINTMap_analysis
        code_returned = MINTMap_analysis(MINTmap_folder, reads, name, num_threads, species, Debug)
        if code_returned:
            functions.time_functions.print_time_stamp(filename_stamp)
            return True
        else:
            print ('** Sample %s failed...' %name)
            return(False)

############################
def MINTMap_analysis(path_folder, reads, name, num_threads, species, Debug):
    
    ## check species
    species_code=""
    if species=="hsa":
        species_code="default"
    else:
        print (colored("** ERROR: Not available yet. No mapping bundle available for species " + species, 'red'))
        exit()
    
    ## debug messages 
    if Debug:    
        HCGB_aes.debug_message("species: " + species, "yellow")
        HCGB_aes.debug_message("species_code: " + species_code, "yellow")
    
    ## ATTENTION: MINTmap needs to chdir to output folder
    path_here = os.getcwd()
    
    ## debug messages 
    if Debug:    
        HCGB_aes.debug_message("path_here: " + path_here, "yellow")
            
    filename_stamp = path_folder + '/.success_mintmap'
    if os.path.isfile(filename_stamp):
        stamp = functions.time_functions.read_time_stamp(filename_stamp)
        print (colored("\tA previous command generated results on: %s [%s -- %s]" %(stamp, name, 'MINTmap call'), 'yellow'))
    else:
        # Call MINTMap_analysis
        codeReturn = MINTmap(reads, path_folder, name, num_threads, species_code, Debug)
        os.chdir(path_here)
        
        if not codeReturn:
            print ('** Sample %s failed...' %name)
            return False

        ## create time stamp        
        functions.time_functions.print_time_stamp(filename_stamp)

    ## Get MINTmap matrix
    MINTmap_matrix_folder = functions.files_functions.create_subfolder("mintmap_parse", path_folder)

    files = os.listdir(path_folder)
    for item in files:
        abs_path_file = os.path.abspath(os.path.join(path_folder, item))
        
        ## debug messages 
        if Debug:    
            HCGB_aes.debug_message("abs_path_file: " + abs_path_file, "yellow")
        
        ## parse them
        if 'countsmeta' in item:
            continue
        if item.endswith('html'):
            continue
        if 'ambigu' in item:
            amb_file = parse_tRF(abs_path_file, name, MINTmap_matrix_folder, 'amb', Debug)        
        elif 'exclu' in item:
            exc_file = parse_tRF(abs_path_file, name, MINTmap_matrix_folder, 'exc', Debug)
    
    ## debug messages 
    if Debug:    
        HCGB_aes.debug_message("exc_file: " + exc_file, "yellow")
        HCGB_aes.debug_message("amb_file: " + amb_file, "yellow")
        
    
    if functions.files_functions.is_non_zero_file(amb_file) and functions.files_functions.is_non_zero_file(exc_file):
        filename_stamp = path_folder + '/.success_all'
        functions.time_functions.print_time_stamp(filename_stamp)
    
    return(True)

##############
def parse_tRF(pathFile, sample_name, matrix_folder, ident, Debug):
    
    ## tsv file name
    tsv_file = os.path.join(matrix_folder, sample_name + "_" + ident + '.tsv')
    
    ## time stamp
    filename_stamp = matrix_folder + '/.success_parse'
    if os.path.isfile(filename_stamp) and functions.files_functions.is_non_zero_file(tsv_file):
        stamp = functions.time_functions.read_time_stamp(filename_stamp)
        print (colored("\tA previous command generated results on: %s [%s -- %s]" %(stamp, sample_name, 'MINTmap - parse'), 'yellow'))
    else:
        ## Open file
        fil = open(tsv_file, 'w')
        string2write = 'UID\tRead\ttRNA\tvariant\tident\texpression\tsoft\n'
        fil.write(string2write)
        ## Read file
        expression_file = open(pathFile)
        expression_text = expression_file.read()
        expression_lines = expression_text.splitlines()
        
        ## debug messages 
        if Debug:    
            HCGB_aes.debug_message("MINTmap file: " + pathFile, "yellow")
        
        for line in expression_lines:
            # ------------------------------ #
            # Example line:
            # tRF-31-87R8WP9N1EWJ0    TCCCTGGTGGTCTAGTGGTTAGGATTCGGCG    5'-tRF    921    7026.67    452.60    na    trna77_GluCTC_6_+_28949976_28950047@1.31.31, trna80_GluCTC_1_-_161417018_161417089@1.31.31
            # ------------------------------ #
            
            if not line.startswith('#'):
                if not line.startswith('License Plate'):
                    UID = line.split('\t')[0]        ## License Plate
                    seq = line.split('\t')[1]        ## tRF sequence
                    variant = line.split('\t')[2]    ## tRF type
                    expression = line.split('\t')[3] ## unnormalized counts
                    ### there are other RPM counts taking into account several things such as total base pairs, reads, etc. We would use raw counts
                    
                    ## Get tRNA name
                    tRNA_name = line.split('\t')[-1].split(',')[0]
                    tRNA_search = re.search(r"trna.{1,3}\_(.{6})\_(.{1,2})\_.*", tRNA_name)
                    tRNA_family = 'na'
                    if tRNA_search:
                        tRNA_family = tRNA_search.group(1)
                        if (tRNA_search.group(2) == 'MT'):
                            tRNA_family = tRNA_family + '_MT'
                        
                    
                    string2write = UID + '\t' + seq + '\t' + tRNA_family +'\t' + variant +'\t' + ident + '\t' + expression + '\tmintmap\n'
                    
                    ## debug messages 
                    if Debug:    
                        HCGB_aes.debug_message(string2write, "yellow")
                    
                    fil.write(string2write)

        fil.close()

        return(tsv_file)

##############
def MINTmap(reads, outpath, name, num_threads, species_code, Debug):
    
    outpath = os.path.abspath(outpath)
    functions.files_functions.create_folder(outpath)
    
    ## change path where the results are required
    os.chdir(outpath)
    
    mintmap_exe = set_config.get_exe("MINTmap", Debug=Debug)
    logfile = os.path.join(outpath, 'MINTmap.log')
    
    ## output
    outpath_file = os.path.join(outpath, name)
    
    if (len(reads) > 1):
        print (colored("** ERROR: Only 1 fastq file is allowed please joined reads before...", 'red'))
        exit()
    
    ## species bundle
    if species_code == "default": 
        ## create command: use default mapping bundle provided with MINTmap 
        cmd = '%s -p %s %s 2> %s' %(mintmap_exe, name, reads[0], logfile)
    else:
        ## create command: use specific mapping bundle path 
        cmd = '%s -p %s -m %s %s 2> %s' %(mintmap_exe, name, species_code, reads[0], logfile)
    
    return(functions.system_call_functions.system_call(cmd))
 


#!/usr/bin/env python3
############################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain   ##
############################################################
## useful imports
import time
import io
import os
import re
import sys
from sys import argv
import subprocess
from termcolor import colored
import concurrent.futures

## import my modules
from XICRA.config import set_config

## import HCGB
from HCGB.functions import system_call_functions, main_functions, time_functions
from HCGB.functions import files_functions, math_functions

## plots
import pandas as pd
import matplotlib

matplotlib.use('agg')
import matplotlib.pyplot as plt
from pandas.plotting import table

#####################
def help_info():
	'''Provide information on RNA biotype analysis'''
	print ("** TODO: Add description on RNAbiotype analysis and details")
	exit()

#####################
def biotype_all(featureCount_exe, path, gtf_file, bam_file, name, threads, Debug, allow_multimap, stranded):
	
	## folder for results
	if not os.path.isdir(path):
		files_functions.create_folder(path)

	out_file = os.path.join(path, 'featureCount.out')
	logfile = os.path.join(path, name + '_RNAbiotype.log')

	filename_stamp_all = path + '/.success_all'
	if os.path.isfile(filename_stamp_all):
		stamp = time_functions.read_time_stamp(filename_stamp_all)
		print (colored("\tA previous command generated results on: %s [%s -- %s]" %(stamp, name, 'RNAbiotype'), 'yellow'))
		return()

	else:
		filename_stamp_featureCounts = path + '/.success_featureCounts'
		if os.path.isfile(filename_stamp_featureCounts):
			stamp = time_functions.read_time_stamp(filename_stamp_featureCounts)
			print (colored("\tA previous command generated results on: %s [%s -- %s]" %(stamp, name, 'featureCounts'), 'yellow'))
		else:
            
			## debugging messages
			if Debug:
				print ("** DEBUG:")
				print ("featureCounts system call for sample: " + name)
				print ("out_file: " + out_file)
				print ("logfile: " + logfile)
		
			## send command for feature count
			## Allow multimapping
			if allow_multimap:
				cmd_featureCount = ('%s -s %s -M -O -T %s -p -t exon -g transcript_biotype -a %s -o %s %s 2> %s' %(
					featureCount_exe, stranded, threads, gtf_file, out_file, bam_file, logfile)
				)
			else:
				cmd_featureCount = ('%s -s %s --largestOverlap -T %s -p -t exon -g transcript_biotype -a %s -o %s %s 2> %s' %(
					featureCount_exe, stranded, threads, gtf_file, out_file, bam_file, logfile)
				)
				
				
			## system call
			cmd_featureCount_code = system_call_functions.system_call(cmd_featureCount, False, True)
			if not cmd_featureCount_code:
				print("** ERROR: featureCount failed for sample " + name)
				exit()
				
			## print time stamp
			time_functions.print_time_stamp(filename_stamp_featureCounts)
		
		## parse results
		(extended_Stats_file, RNAbiotypes_stats_file) = parse_featureCount(out_file, path, name, bam_file, Debug)
		
		## debugging messages
		if Debug:
			print ("** DEBUG:")
			print ("extended_Stats: " + extended_Stats_file)
			print (main_functions.get_data(extended_Stats_file, '\t', 'header=None'))
			print ("RNAbiotypes_stats: " + RNAbiotypes_stats_file)
			print (main_functions.get_data(RNAbiotypes_stats_file, '\t', 'header=None'))

	return ()

#######################################################################
def parse_featureCount(out_file, path, name, bam_file, Debug):
	"""
	Parses featureCount results for RNAbiotype analysis.
	
	:param out_file: Name provided to featureCount for output results.
	:param path:
	:param name:
	
	
	"""

	## file names
	out_tsv_file_name = out_file + '.tsv'
	RNA_biotypes_file_name = os.path.join(path, name + '_RNAbiotype.tsv')

	##
	filename_stamp_parse = path + '/.success_parse'
	if os.path.isfile(filename_stamp_parse):
		stamp = time_functions.read_time_stamp(filename_stamp_parse)
		print (colored("\tA previous command generated results on: %s [%s -- %s]" %(stamp, name, 'parse results'), 'yellow'))
	else:
	
		## debugging messages
		if Debug:
			print ("** DEBUG:")
			print ("Parse results for sample: " + name)
			
		## parse results
		out_tsv_file = open(out_tsv_file_name, 'w')
		RNA_biotypes_file = open(RNA_biotypes_file_name, 'w')
		tRNA_count = 0
		
		##########################################
		### read count file
		##########################################
		count_file = open(out_file)
		count_file_text = count_file.read()
		count_file_lines = count_file_text.splitlines()	
	
		for line in count_file_lines:
			if line.startswith('#'):
				continue
			elif line.startswith('Geneid'):
				continue
			else:
				ID = line.split('\t')[0]
				count = int(line.split('\t')[-1])
				string2write_raw = "%s\t%s\n" %(ID, count)
				out_tsv_file.write(string2write_raw)
	
				tRNA_search = re.search(r".*tRNA", ID)
				if tRNA_search:
					tRNA_count = int(tRNA_count) + int(count)				
				elif (count > 0):
					RNA_biotypes_file.write(string2write_raw)
		
		## count and summary tRNA
		string2write = "tRNA\t%s\n" %tRNA_count
		RNA_biotypes_file.write(string2write)
		RNA_biotypes_file.close()
		
		##########################################
		### read summary count file
		##########################################
		summary_count_file = open(out_file + '.summary')
		summary_count_file_text = summary_count_file.read()
		summary_count_file_lines = summary_count_file_text.splitlines()	
	
		for line in summary_count_file_lines:
			if line.startswith('Status'):
				continue
			elif line.startswith('Assigned'):
				continue
			else:
				## adds Unassigned_Ambiguity
				## adds Unassigned_NoFeatures
				ID = line.split('\t')[0]
				count = int(line.split('\t')[-1])
	
				## skip empty entries
				if count == 0:
					continue
				string2write_raw = "%s\t%s\n" %(ID, count)
				out_tsv_file.write(string2write_raw)
	
		##########################################
		## get mapping statistics according to mapping software
		##########################################
		count_multi = 0
		count_unmap = 0
		mapping_folder = os.path.dirname(bam_file)
		mapping_stats = mapping_folder + '/Log.final.out'
		
		## -------------------------------- ##
		### STAR mapping		
		## -------------------------------- ##
		if files_functions.is_non_zero_file(mapping_stats):
			## debugging messages
			if Debug:
				print ("** DEBUG:")
				print ("STAR mapping available for sample: " + name)
				print ("mapping_folder: " + mapping_folder)
	
			mapping_stats_file = open(mapping_stats)
			mapping_stats_file_text = mapping_stats_file.read()
			mapping_stats_file_lines = mapping_stats_file_text.splitlines()	
	
			for line in mapping_stats_file_lines:
				multi_search = re.search(r".*Number of reads mapped to", line)
				unmap_search = re.search(r".*unmapped.*", line)
				input_search = re.search(r".*input reads.*", line)
			
				if input_search:
					total_input_reads = int(line.split('\t')[-1])
	
				if multi_search:
					count_tmp = int(line.split('\t')[-1])
					count_multi = count_multi + count_tmp
	
				elif unmap_search:
					perc_tmp = line.split('\t')[-1]
					count_reads = math_functions.percentage(perc_tmp, total_input_reads)
					count_unmap = count_unmap + count_reads
		else:
	
			## -------------------------------- ##
			## tophat
			## -------------------------------- ##
	
			mapping_stats = mapping_folder + '/align_summary.txt' 
			count_map = 0
			total_input_reads = 0
			
			if files_functions.is_non_zero_file(mapping_stats):
				## debugging messages
				if Debug:
					print ("** DEBUG:")
					print ("tophat mapping available for sample: " + name)
					print ("mapping_folder: " + mapping_folder)
				
				mapping_stats_file = open(mapping_stats)
				mapping_stats_file_text = mapping_stats_file.read()
				mapping_stats_file_lines = mapping_stats_file_text.splitlines()	
	
				for line in mapping_stats_file_lines:
					map_search2 = re.search(r"Aligned.*\:\s+(\d+).*", line)
					input_search2 = re.search(r".*Input.*\:\s+(\d+).*", line)
					if input_search2:
						total_input_reads = input_search2.group(1)
					if map_search2:
						count_map = map_search2.group(1)
		
				####
				count_unmap = int(total_input_reads) - int(count_map)
	
			else:
				## other
				print ("Neither tophat or STAR..., no mapping statistics")
	
		### print mapping stats
		string2write_unmap = "unmapped\t%s\n" %count_unmap
		out_tsv_file.write(string2write_unmap)
		
		## close files
		out_tsv_file.close()
		summary_count_file.close()
		mapping_stats_file.close()
		count_file.close()
		## print timestamp
		time_functions.print_time_stamp(filename_stamp_parse)

	return(out_tsv_file_name, RNA_biotypes_file_name)

#######################################################################
def RNAbiotype_module_call(samples_dict, output_dict, gtf_file, Debug, max_workers_int, threads_job, multimapping, stranded):
	"""
	Create RNAbiotype analysis for each sample and create summary plots
	
	:param samples_dict: Dictionary containing sample IDs as keys and bam files as values
	:param output_dict: Dictionary containing sample IDs as keys and output folder as values
	:param gtf_file: Gene annotation file for the reference genome used.
	:param threads: Number of threads to use.
	:param Debug: True/False for debugging messages
	"""
	
	## get bin
	featureCount_exe = set_config.get_exe('featureCounts')

	## send for each sample
	with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers_int) as executor:
		commandsSent = { executor.submit(biotype_all, featureCount_exe, 
										output_dict[sample], gtf_file, bam_files, 
										sample, threads_job, Debug, multimapping, stranded): sample for sample, bam_files in samples_dict.items() }
	
		for cmd2 in concurrent.futures.as_completed(commandsSent):
			details = commandsSent[cmd2]
			try:
				data = cmd2.result()
			except Exception as exc:
				print ('***ERROR:')
				print (cmd2)
				print('%r generated an exception: %s' % (details, exc))

	##
	## plot results
	for name, folder in output_dict.items():
		RNAbiotypes_stats_file = os.path.join(folder, name + '_RNAbiotype.tsv')
		if files_functions.is_non_zero_file(RNAbiotypes_stats_file):
			pie_plot_results(RNAbiotypes_stats_file, name, folder, Debug)
			
	return()

############################################################
def generate_matrix(dict_files):
	######################################################
	### For a dictionary containing names as keys and files as values, 
	###	generates a count matrix with the classification from featurecounts
	###		
	### It returns a dataframe containing for each index generated
	### count values for each sample (sample_name) in columns
	###
	#########################################################
	all_data = pd.DataFrame()
	for key,values in dict_files.items():
		print ('+ Reading information from sample: ', key)	
		data = pd.read_csv(values, sep='\t', header=None, names=['RNAbiotypes', key])
		
		## skip if file is empty
		if data.empty:
			continue

		## get info, generate unique name and merge for samples
		data = data.set_index('RNAbiotypes')
		all_data = pd.concat([all_data, data], axis=1, sort=True)
	
	return (all_data)
	##

#######################################################################
def pie_plot_results(RNAbiotypes_stats_file, name, folder, Debug):
	
	##
	filename_stamp_plot = folder + '/.success_plot'
	if os.path.isfile(filename_stamp_plot):
		stamp = time_functions.read_time_stamp(filename_stamp_plot)
		print (colored("\tA previous command generated results on: %s [%s -- %s]" %(stamp, name, 'plot results'), 'yellow'))
	else:
		
		# PLOT and SHOW results
		RNAbiotypes_stats = main_functions.get_data(RNAbiotypes_stats_file, '\t', 'header=None')
	
		# create plot
		plt.figure(figsize=(16,8))
		df_genetype_2 = pd.DataFrame({'Type':RNAbiotypes_stats[0], 
									'Count':RNAbiotypes_stats[1]}).sort_values(by=['Count'])
	
		## get total count
		df_genetype_ReadCount_sum = df_genetype_2['Count'].sum()
	
		## filter 1% values
		minimun = df_genetype_ReadCount_sum * 0.01
		df_genetype_filter_greater = df_genetype_2[ df_genetype_2['Count'] >= minimun ]
		df_genetype_filter_smaller = df_genetype_2[ df_genetype_2['Count'] < minimun ]
	
		## create %values
		df_genetype_2['Percentage'] = (df_genetype_2['Count']/df_genetype_ReadCount_sum*100).round(3)
		
		## merge and generate Other class
		df_genetype_filter_smaller_sum = df_genetype_filter_smaller['Count'].sum() ## total filter smaller
		df_genetype_filter_greater2 = df_genetype_filter_greater.append({
			'Count':df_genetype_filter_smaller_sum, 
			'Type':'Other'}, ignore_index=True)
	
		## Create Pie Plot
		ax1 = plt.subplot(121, aspect='equal')
		df_genetype_filter_greater2.plot.pie(
			y = 'Count', 
			ax=ax1, 
			autopct='%1.2f%%', 
			shadow=False, 
			labels=df_genetype_filter_greater2['Type'], 
			legend = False)
	
		# plot table
		ax2 = plt.subplot(122)
		plt.axis('off')
		tbl = ax2.table(
			cellText=df_genetype_2.values, 
			colLabels=df_genetype_2.columns,
			loc='center', rowLoc='left', cellLoc='center', 
			)
		tbl.auto_set_font_size(True)
		#tbl.set_fontsize(12)
		tbl.scale(1.1,1.1)
	
		## set PDF name
		name_figure = os.path.join(folder, name + '_RNAbiotypes.pdf')
	
		## generate image
		plt.savefig(name_figure)		
		plt.close(name_figure)
		plt.close()
		
		## print time stamps
		time_functions.print_time_stamp(filename_stamp_plot)
		filename_stamp_all = folder + '/.success_all'
		time_functions.print_time_stamp(filename_stamp_all)
		
#######################################################################
def main():
	
	## ARGV
	if len (sys.argv) < 6:
		print ("\nUsage:")
		print ("python3 %s bam_file folder gtf_file threads name featureCount_bin multimapping[True/False]\n" %os.path.realpath(__file__))
		exit()
	
	bam_file = os.path.abspath(argv[1])
	folder = os.path.abspath(argv[2])
	gtf_file = os.path.abspath(argv[3])
	threads = argv[4]
	name = argv[5]
	featureCount_exe = argv[6]
	multimapping= argv[7]

	## Debug
	Debug=True
	
	## variables
	biotype_all(featureCount_exe, folder, gtf_file, bam_file, name, threads, Debug, multimapping)
	## plot results
	RNAbiotypes_stats_file = os.path.join(folder, name + '_RNAbiotype.tsv')
	if files_functions.is_non_zero_file(RNAbiotypes_stats_file):
		pie_plot_results(RNAbiotypes_stats_file, name, folder, Debug)
			
	
######
if __name__== "__main__":
	main()
	



#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez                                      ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain ##
##########################################################
"""
Trimms sequence adapters within fastq reads.
"""
## import useful modules
import os
import sys
import re
import time
from io import open
import shutil
import concurrent.futures
from termcolor import colored

## import my modules
from XICRA.scripts import multiQC_report
from XICRA.scripts import cutadapt_caller
from XICRA.config import set_config
from XICRA.modules import help_XICRA
from XICRA.modules import qc
from HCGB import functions
from HCGB import sampleParser

##############################################
def run_trimm(options):

    ## init time
    start_time_total = time.time()

    ##################################
    ### show help messages if desired    
    ##################################
    if (options.help_format):
        ## help_format option
        help_XICRA.help_fastq_format()
    elif (options.help_trimm_adapters):
        ## help on trimm adapters
        help_XICRA.print_help_adapters()
        exit()
    elif (options.help_project):
        ## information for project
        help_XICRA.project_help()
        exit()
    elif (options.help_multiqc):
        ## information for Multiqc
        help_XICRA.multiqc_help()
        exit()
        
    ## debugging messages
    global Debug
    if (options.debug):
        Debug = True
    else:
        Debug = False
        
    ### set as default paired_end mode
    if (options.single_end):
        options.pair = False
    else:
        options.pair = True
    
    functions.aesthetics_functions.pipeline_header('XICRA')
    functions.aesthetics_functions.boxymcboxface("Trimming samples")
    print ("--------- Starting Process ---------")
    functions.time_functions.print_time()

    ## absolute path for in & out
    input_dir = os.path.abspath(options.input)
    outdir=""

    ## set mode: project/detached
    if (options.detached):
        outdir = os.path.abspath(options.output_folder)
        options.project = False
    else:
        options.project = True
        outdir = input_dir        
    
    # Trimming adapters

    ## check adapters provided
        ## options.adapters_a
        ## options.adapters_A
        ## options.extra
        
    ## no adapters provided
    if (not options.adapters_a and not options.adapters_A and not options.extra):
        print (colored("** ERROR: No adapter trimming options provided...", 'red'))
        print ("Please provide any option")
        exit()
    
    ## create dictionary with 
    adapters_dict = {}
    if (options.adapters_a):
        adapters_dict['adapter_a'] = options.adapters_a
    
    if (options.adapters_a):
        adapters_dict['adapter_A'] = options.adapters_A
    
    ## set default
    #if not options.min_len_read:
    #    options.min_len_read=15
    
    ## get files
    print ('+ Getting files from input folder... ')
    print ('+ Mode: fastq.\n+ Extension: ')
    print ("[ fastq, fq, fastq.gz, fq.gz ]\n")
    pd_samples_retrieved = sampleParser.files.get_files(options, input_dir, "fastq", ("fastq", "fq", "fastq.gz", "fq.gz"), options.debug)
    
    ## debug message
    if (Debug):
        print (colored("**DEBUG: pd_samples_retrieve **", 'yellow'))
        print (pd_samples_retrieved)

        print (colored("**DEBUG: adapters_dict **",'yellow'))
        print (adapters_dict)

    ## generate output folder, if necessary
    print ("\n+ Create output folder(s):")
    if not options.project:
        functions.files_functions.create_folder(outdir)
    ## for samples
    outdir_dict = functions.files_functions.outdir_project(outdir, options.project, pd_samples_retrieved, "trimm", options.debug)
    
    ## optimize threads
    name_list = set(pd_samples_retrieved["new_name"].tolist())
    threads_job = functions.main_functions.optimize_threads(options.threads, len(name_list)) ## threads optimization
    max_workers_int = int(options.threads/threads_job)

    ## debug message
    if (Debug):
        print (colored("**DEBUG: options.threads " +  str(options.threads) + " **", 'yellow'))
        print (colored("**DEBUG: max_workers " +  str(max_workers_int) + " **", 'yellow'))
        print (colored("**DEBUG: cpu_here " +  str(threads_job) + " **", 'yellow'))

    print ("+ Trimming adapters for each sample retrieved...")    
    
    # Group dataframe by sample name
    sample_frame = pd_samples_retrieved.groupby(["new_name"])
    
    ## send for each sample
    with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers_int) as executor:
        commandsSent = { executor.submit(cutadapt_caller.caller, sorted(cluster["sample"].tolist()), 
                                         outdir_dict[name], name, threads_job, 
                                         options.min_read_len, Debug, adapters_dict, options.extra): name for name, cluster in sample_frame }

        for cmd2 in concurrent.futures.as_completed(commandsSent):
            details = commandsSent[cmd2]
            try:
                data = cmd2.result()
            except Exception as exc:
                print ('***ERROR:')
                print (cmd2)
                print('%r generated an exception: %s' % (details, exc))

    print ("\n\n+ Trimming samples has finished...")
    
    ## functions.time_functions.timestamp
    start_time_partial = functions.time_functions.timestamp(start_time_total)

    ## get files generated and generate symbolic link
    if not options.project:
        dir_symlinks = functions.files_functions.create_subfolder('link_files', outdir)
        files2symbolic = []
        folders = os.listdir(outdir)

        ## debug message
        if (Debug):
            print (colored("**DEBUG: generate symbolic links for each file in " + dir_symlinks + "**", 'yellow'))
        
        for fold in folders:
            if fold.endswith(".log"):
                continue
            else:
                this_folder = outdir + '/' + fold
                subfiles = os.listdir(this_folder)
                for files in subfiles:
                    files_search = re.search(r".*trim_R\d{1}.*", files) ## only paired-end. Todo: single end
                    if files_search:
                        files2symbolic.append(this_folder + '/' + files)
    
        functions.files_functions.get_symbolic_link(files2symbolic, dir_symlinks)

    if (options.skip_report):
        print ("+ No report generation...")
    else:
        print ("\n+ Generating a report using MultiQC module.")
        outdir_report = functions.files_functions.create_subfolder("report", outdir)
    
        ## call multiQC report module
        givenList = [ v for v in outdir_dict.values() ]
        my_outdir_list = set(givenList)
        
        ## debug message
        if (Debug):
            print (colored("\n**DEBUG: my_outdir_list for multiqc report **", 'yellow'))
            print (my_outdir_list)
            print ("\n")

        trimm_report = functions.files_functions.create_subfolder("trimm", outdir_report)
        multiQC_report.multiQC_module_call(my_outdir_list, "Cutadapt", trimm_report,"")
        print ('\n+ A summary HTML report of each sample is generated in folder: %s' %trimm_report)
        
        ## QC analysis for trimmed reads
        if (Debug):
            print (colored("** Beginning FAStQC analysis **", 'red'))

        ## functions.time_functions.timestamp
        start_time_partial = functions.time_functions.timestamp(start_time_partial)

    ## create FASTQC calling for trimmed reads
    pd_samples_retrieved_trimmed = sampleParser.files.get_files(options, input_dir, "trim", ['_trim'], options.debug)
    qc.fastqc(pd_samples_retrieved_trimmed, outdir, options, "trimmed", start_time_partial, Debug)
        
    print ("\n*************** Finish *******************")
    start_time_partial = functions.time_functions.timestamp(start_time_total)
    print ("\n+ Exiting trimm module.")
    exit()
    


#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez										##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain	##
##########################################################
"""
Provides configuration for the pipeline.
"""
## useful imports
import os
import io
import sys
import re
import shutil
from io import open
from sys import argv
import subprocess
from termcolor import colored
from distutils.version import LooseVersion
import pkg_resources

## import my modules
from HCGB import functions
from XICRA.config import extern_progs

################
## Software
################

##################
def get_exe(prog, Debug=False, Return_Version=False):
	"""Return absolute path of the executable program requested.

	Given a program name it returns its executable to be called. It has to fulfilled a minimum version specified.

	:param prog: Software name
	:type prog: string
	:returns: Absolute path for the executable requested
	:warning: if no executable available in system ``$PATH`` or not fulfilling the expected version.

	.. attention:: Be aware of Copyright

		The code implemented here was retrieved and modified from ARIBA (https://github.com/sanger-pathogens/ariba)

		Give them credit accordingly.

	"""
	exe = ""
	if prog in os.environ: 
		exe = os.environ[env_var] ## python environent variables
	else:
		exe = extern_progs.return_defatult_soft(prog) ## install in the system

	## get paths
	exe_path_tmp = my_which(exe)

	## debug message
	if (Debug):
		print(colored("** Debug: exe: %s" %exe,'yellow'))
		print(colored("** Debug: exe_path_tmp: %s" %exe_path_tmp,'yellow'))

	## get min_version
	min_version = extern_progs.return_min_version_soft(prog)
	
	## debug message
	if (Debug):
		print(colored("** Debug: min_version: %s" %min_version,'yellow'))

	## return if not available
	## no min version available
	if min_version == 'na':
		if exe_path_tmp:
			if (Return_Version):
				return (exe_path_tmp[0], '') ## return first item
			else:
				return (exe_path_tmp[0]) ## return first item
	
	## not installed in path
	if exe_path_tmp is None or len(exe_path_tmp) == 0:
		if (Return_Version):
			print(colored("\n**ERROR: Software %s could not be found." % prog,'red'))
			exit()
			return('ERROR', 'n.a.')
		else:
			print(colored("\n**ERROR: Software %s could not be found." % prog,'red'))
			exit()
			return('ERROR')
		

	## Loop for all possibilities
	for p in exe_path_tmp:
		prog_ver = get_version(prog, p, Debug=Debug)

		if (Debug):
			print (colored("** Debug: Software: %s\nPath: %s\nVersion: %s" %(prog, p, prog_ver), 'yellow'))

		if (prog_ver == 'n.a.'):
			continue

		if LooseVersion(prog_ver) >= LooseVersion(min_version):
			if (Return_Version):
				return (p, prog_ver)
			else:
				return (p)


	print(colored("\n**ERROR: Software %s version smaller than minimum version expected %s." %(prog,min_version),'red'))
	exit()

################
def access_check(fn, mode=os.F_OK | os.X_OK):
	"""Check executable permission

	This function checks whether a given path is a folder or file and if it is 
	executable and accessible. It also works if a java jar file provided.

	:param fn: Absolute path file
	:param mode: Value to pass as the mode parameter of access()

	:type fn: string
	:type mode: string

	`mode` defaults to:

		- os.F_OK: Value to pass as the mode parameter of access() to test the existence of path.

		- os.X_OK: Value to include in the mode parameter of access() to determine if path can be executed.

	.. attention:: Be aware of Copyright

		The code implemented here was retrieved and modified from shutil (https://github.com/python/cpython/blob/master/Lib/shutil.py).

		Give them credit accordingly.

		We modified the code to work if java jar files provided.
	"""
	## the original code belongs to shutil, slightly modified here
	# https://github.com/python/cpython/blob/master/Lib/shutil.py

	#if os.path.isdir(fn):
	#	return False

	if os.path.exists(fn):
		if fn.endswith('.jar'):
			return True

		if os.access(fn, mode):
			return True

#################
def my_which(cmd):
	"""Return the absolute path to the executable

	Given a command return the absolute path(s), if any.

	:param cmd: Software command name 
	:returns: List of absolute paths(s) of the given command.

	.. attention:: Be aware of Copyright

		The code implemented here was retrieved and modified from shutil (https://github.com/python/cpython/blob/master/Lib/shutil.py).

		Give them credit accordingly.

		We modified the code to return multiple paths in a list if available different installed binaries in $PATH.

	"""
	# If we're given a path with a directory part, look it up directly rather
	# than referring to PATH directories. This includes checking relative to the
	# current directory, e.g. ./script
	if os.path.dirname(cmd):
		if access_check(cmd):
			return cmd
		return None

	use_bytes = isinstance(cmd, bytes)

	path=None

	if path is None:
		path = os.environ.get("PATH", None)

	if path is None:
		try:
			path = os.confstr("CS_PATH")
		except (AttributeError, ValueError):
			# os.confstr() or CS_PATH is not available
			path = os.defpath
		# bpo-35755: Don't use os.defpath if the PATH environment variable is
		# set to an empty string

	# PATH='' doesn't match, whereas PATH=':' looks in the current directory
	if not path:
		return None

	if use_bytes:
		path = os.fsencode(path)
		path = path.split(os.fsencode(os.pathsep))
	else:
		path = os.fsdecode(path)
		path = path.split(os.pathsep)

	# On other platforms you don't have things like PATHEXT to tell you
	# what file suffixes are executable, so just pass on cmd as-is.
	files = [cmd]

	return_paths = [] ## modification
	seen = set()
	for dir in path:
		normdir = os.path.normcase(dir)
		#print ("Normdir: ", normdir)
		if not normdir in seen:
			seen.add(normdir)
			for thefile in files:
				name = os.path.join(dir, thefile)
				#print ("Name: ", name)
				if access_check(name):
					## return (name) ## previously, it would only return the first item
					return_paths.append(name) ## modification

	if (len(return_paths) >= 1):
		return return_paths
	else:
		return None

##################
def get_version(prog, path, Debug=False):
	"""Get version of software

	Given a program name and expected path, tries to determine its version.

	:param prog: Program name
	:param path: Absolute path
	:param Debug: True/False

	:type prog: string
	:type path: string 
	:type Debug: bool

	:returns: String containing version. Returns NA message if no found and raises attention error message.

	.. attention:: Be aware of Copyright

		The code implemented here was retrieved and modified from ARIBA (https://github.com/sanger-pathogens/ariba)

		Give them credit accordingly.
	"""

	## read dependencies information
	dependencies_pd = extern_progs.read_dependencies()

	## get information for prog
	regex = re.compile(dependencies_pd.loc[prog, 'get_version'])
	args = dependencies_pd.loc[prog, 'version_cmd']
	cmd = path + ' ' + args

	## debug messages
	if (Debug):
		print(colored("** Debug: regex: %s" %regex,'yellow'))
		print(colored("** Debug: args: %s" %args, 'yellow'))

	if prog == 'sRNAbench' or prog == "miraligner":
		java_bin = get_exe('java', Debug=Debug)
		java_jar = java_bin + ' -jar ' + path + ' ' + args
		cmd_output = subprocess.Popen(java_jar, shell=True, stdout=subprocess.PIPE,stderr=subprocess.PIPE).communicate()
	else:
		cmd_output = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()

	## decode command
	cmd_output = functions.main_functions.decode(cmd_output[0]).split('\n')[:-1] + functions.main_functions.decode(cmd_output[1]).split('\n')[:-1]

	## debug messages
	if (Debug):
		print(colored("** Debug: cmd_output:\n %s" %cmd_output,'yellow'))

	## retrieve version information
	for line in cmd_output:
		hits = regex.search(line)
		if (Debug):
			print (hits)
		if hits:
			return hits.group(1)

	if Debug:
		print (colored('Attention: I tried to get the version of ' + prog + ' with: "' + cmd + '" and the output didn\'t match this regular expression: "' + regex.pattern + '"', 'red'))

	return("n.a.")

def check_dependencies(Debug):
	"""
	Check if available the different software required for ``XICRA`` execution.
	
	Using the function :func:`XICRA.config.extern_progs.read_dependencies` the 
	information for all the dependencies is retrieved from file :file:`XICRA/config/software/dependencies.csv`.
	
	For each software, the function :func:`XICRA.config.set_config.get_exe` retrieves
	whether it is installed in the system or not and its version and it is check using 
	:func:`XICRA.config.set_config.check_install`. 
	
	:param Debug: True/False for debugging messages 
	:type Debug: boolean
	
	:returns: Print messages and information	
	"""
	
	## read dependencies information
	dependencies_pd = extern_progs.read_dependencies()

	for soft, row in dependencies_pd.iterrows():
		soft_name = row['soft_name']
		min_version = row['min_version']
		
		## debug messages
		if (Debug):
			print ("Software:", soft)
			print ("Soft name:", soft_name)
			print ("Min_Version:", min_version)
		
		## get executable
		(soft_path, installed) = get_exe(soft, Debug=Debug, Return_Version=True)
		
		## debug messages
		if (Debug):
			print ("Soft Path: ", soft_path)
			print ("Version installed:", installed)
			
		## check if installed
		if (min_version == 'na'):
			message = check_install_module('ok', soft_name, min_version, 'Software')
		else:
			message = check_install_module(installed, soft_name, min_version, 'Software')

		if (message == 'OK'):
			continue
		else:
			print ("+ Please install manually software: ", soft_name, " to continue with XICRA\n\n")
	
################
## Python
################
def get_python_packages(Debug):
	"""
	Retrieves the version of the python packages installed in the system.

	It retrieves the dependencies name conversion from file :file:`XICRA/config/python/module_dependencies.csv`
	using function :func:`XICRA.config.extern_progs.file_list` and :func:`XICRA.scripts.functions.main_functions.get_data`.
	For each module it retrieves the package version installed in the system using 
	:func:`XICRA.config.set_config.check_package_version`.	

	:returns: Dictionary containing for each python module (key) the installed version (value).
	"""

	## get import names for packages:
	## some modules do not have the same name when install from pip and called from import
	file_module_dependecies = extern_progs.file_list("python_requirements")
	module_dependencies = functions.main_functions.file2dictionary(file_module_dependecies, ',')

	my_packages_installed = {}
	for each in module_dependencies:
		module_name = module_dependencies[each]
		installed = check_package_version(each, Debug) ## check version installed in system
		my_packages_installed[each] = installed

	return (my_packages_installed)

##################
def check_python_packages(Debug):
	"""
	This functions checks whether the packages installed in the system fulfilled the 
	minimum version specified in the configuration folder. 

	It uses function :func:`XICRA.config.set_config.get_python packages` to
	retrieve the version of the python packages installed in the system. Then it uses
	:func:`XICRA.config.extern_progs.min_python_module_version` to retrieve the minimum
	version specified. It compares them using function :func:`XICRA.config.set_config.check_install_module`.

	:param Debug: True/False for debugging messages
	:type Debug: boolean

	:returns: Print messages if packages are installed.
	"""
	## get python packages installed
	my_packages_installed = get_python_packages(Debug)

	## debug messages
	if (Debug):
		print ("my_packages_installed :: ")
		print (my_packages_installed)

	## min versions for packages
	my_packages_requirements = extern_progs.min_python_module_version()

	## debug messages
	if (Debug):
		print ("my_packages_requirements")
		print (my_packages_requirements)

	## check each package
	for each in my_packages_requirements:
		## get min version
		min_version = my_packages_requirements[each]

		## get version installed in system
		installed = my_packages_installed[each] 

		## debug messages
		if (Debug):
			print ("Module:", each)
			print ("Min_Version:", min_version)
			print ("Version installed:", installed)
			
		## check if installed
		message = check_install_module(installed, each, min_version, 'Module')

		if (message == 'OK'):
			continue
		else:
			print ("+ Please install manually package: ", each, " to continue with XICRA\n\n")
			#print ("pip install %s" %each)

################
def check_package_version(package, Debug):
	"""
	Retrieve python package version installed

	This is a modification of the original code from ARIBA (https://github.com/sanger-pathogens/ariba). 
	It basically uses pkg_resources.get_distribution(), pkg_resources.resource_filename() or imports module
	and retrieves version from __version__ variable.

	:param package: Python package name 
	:param Debug: True/False for debugging messages

	:type package: string
	:type Debug: boolean

	:returns: Version retrieved

	.. attention:: Be aware of Copyright

		The code implemented here was retrieved and modified from ARIBA (https://github.com/sanger-pathogens/ariba)

		Give them credit accordingly.
	"""

	try:
		version = pkg_resources.get_distribution(package).version
		if (Debug):
			print ("Method: pkg_resources.get_distribution(package).version")
	except:
		try:
			exec('import ' + package)
			version = eval(package + '.__version__')
			if (Debug):
				print ("Method: exec('import ' + package); version = eval(package + '.__version__')")
		except:
			try:
				if (Debug):
					print ("Method: pkg_resources.resource_filename(package, 'version.py')")
				version = pkg_resources.resource_filename(package, 'version.py')
			except:
				version = 'n.a.'
	if (Debug):
		print ('Package:', package)
		print ('Version:', version)
	return(version)
################

################
## R
################
def get_R_packages():
	dep_file = os.path.abspath(os.path.join(os.path.dirname( __file__ ), 'R', 'R_dependencies.csv'))
	dep_file_data = functions.main_functions.get_data(dep_file, ',', 'index_col=0')
	return (dep_file_data)

################
def check_R_packages(Debug):
	
	packages = get_R_packages()
	check_install_system = os.path.abspath(os.path.join(os.path.dirname( __file__ ), 'R', 'check_install_system.R'))
	R_script_exe = get_exe('Rscript')
	
	for index,row in packages.iterrows():
		## debugging messages
		if Debug:
			print ('\n+ Check package: ', index)
			print('+ Source: ', row['source'])
		
		cmd_check = R_script_exe + ' ' + check_install_system + ' -l ' + index
		code = functions.system_call_functions.system_call(cmd_check, message=False, returned=False)
		if (code=='OK'):
			check_install_module('1', index, '0', 'package')
		else:
			check_install_module('0', index, '1', 'System package')

			## check if installed in path
			cmd_check_path = R_script_exe + ' ' + check_install_path + ' -l ' + index + ' -p ' + install_path
			code2 = functions.system_call_functions.system_call(cmd_check_path, message=False, returned=False)

			if (code2):
				check_install_module('1', index, '0', 'Install path package')
			else:
				check_install_module('0', index, '1', 'Install path package')
				print ("Please install module %s manually to continue with XICRA" %index)
				
		
################
## Miscellaneous
################
def print_module_comparison(module_name, message, color, tag):
	"""
	Creates print message for a given module, version, message

	:param module_name: Name of the module
	:param message: Message to include in the print message: OK | FAILED | NOT FOUND
	:param color: Print message color: green | orange | red
	:param tag: Tag to include: Module, package, software

	:type module_name: string
	:type message: string 
	:type color: string
	:type tag: string

	:returns: Print message
	"""
	print (colored("{:.<15}{:.>15}".format("%s: %s" %(tag, module_name), "[ %s ]" %message), color))

#########

##################

def check_install_module(installed, module_name, min_version, tag):
	"""
	Checks modules installation 

	Checks whether a module is installed and fulfilling requirements. 	
	It prints messages using :func:`XICRA.config.set_config.print_module_comparison`.

	:param installed: Version string of the module installed.
	:param module_name: Module name
	:param min_version: Version string for the minimum version required

	:type installed: string
	:type module_name: string 
	:type min_version: string
	"""
	 ## Not installed
	if (installed == 'n.a.' or not installed):
		message = 'NOT FOUND'
		color = 'red'
		print_module_comparison(module_name, message, color, tag)

	# check version
	elif LooseVersion(installed) >= LooseVersion(min_version):
		message = 'OK'
		color = 'green'
		print_module_comparison(module_name, message, color, tag)

	# check version
	elif (installed == 'ok'):
		message = 'OK'
		color = 'green'
		print_module_comparison(module_name, message, color, tag)

	else:
		message = 'FAILED'
		color = 'yellow'
		print_module_comparison(module_name, message, color, tag)

	## return message
	return (message)

#########	



#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez, Marta Lopez & Lauro Sumoy           ##
## Copyright (C) 2019-2021 Lauro Sumoy Lab, IGTP, Spain ##
##########################################################
'''
Calls miraligner to create miRNA profile
'''
## useful imports
import time
import io
import os
import re
import sys
from sys import argv
from io import open
from termcolor import colored

## import my modules
from HCGB.functions import fasta_functions
from HCGB import functions
from XICRA.config import set_config

###############       
def miraligner_caller(reads, sample_folder, name, threads, database, species, Debug):
    """Passes the sample information to be analyzed by miraligner().

    Checks if the computation has been performed before. If not, it calls
    miraligner().
    
    :param reads: file with sample reads
    :param sample_folder: output folder
    :param name: sample name
    :param threads: selected threads (by defoult 2)
    :param database: path to store miRNA annotation files downloaded
    :param species: species tag ID. Default: hsa (Homo sapiens)
    :param Debug: display complete log.


    :returns: True/False
    """
    # check if previously generated and succeeded
    filename_stamp = sample_folder + '/.success'
    if os.path.isfile(filename_stamp):
        stamp = functions.time_functions.read_time_stamp(filename_stamp)
        print (colored("\tA previous command generated results on: %s [%s -- %s]" %(stamp, name, 'miraligner'), 'yellow'))
    else:
        # Call miralinger
        code_returned = miraligner(reads, sample_folder, name, database, species, Debug)
        if code_returned:
            functions.time_functions.print_time_stamp(filename_stamp)
        else:
            print ('** Sample %s failed...' %name)
            return(False)
        
    return(True)

###############       
def miraligner (reads, outpath, file_name, database, species, Debug):
    """Executes the miraligner analyis of the sample

    Checks if the reads are joined and builds the miraligner command to execute it. 
    
    :param reads: file with sample reads
    :param outpath: output folder
    :param file_name: sample name
    :param database: path to store miRNA annotation files downloaded
    :param species: species tag ID. Default: hsa (Homo sapiens)
    :param Debug: display complete log.


    :returns:  The miraligner output
    """    
    miraligner_exe = set_config.get_exe("miraligner", Debug=Debug)
    logfile = os.path.join(outpath, 'miraligner.log')
    
    ## output
    outpath_file = os.path.join(outpath, file_name)
    
    if (len(reads) > 1):
        print (colored("** ERROR: Only 1 fastq file is allowed please joined reads before...", 'red'))
        exit()
    
    ## create tabular information of reads
    tabular_info = os.path.join(outpath, file_name + '-tab.freq.txt')
    fasta_functions.reads2tabular(reads[0], tabular_info)
    
    ## create command 
    java_exe = set_config.get_exe('java', Debug=Debug)
    cmd = '%s -jar %s -db %s -sub 1 -add 3 -trim 3 -s %s -i %s -o %s 2> %s' %(
        java_exe, miraligner_exe, database, species, tabular_info, outpath_file, logfile)
    
    return(functions.system_call_functions.system_call(cmd))



__all__ = [
    'fastqc_caller',
    'multiQC_report',
    'generate_DE',
    'RNAbiotype',
    'mapReads',
    'cutadapt_caller',
    'mirtop_caller',
    'optimir_caller',
    'miraligner_caller',
    'MINTMap_caller'
    
]

from XICRA.scripts import *



#!/usr/bin/env python3
############################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain   ##
############################################################
import time
import io
import os
import re
import sys
from io import open
from sys import argv
import pandas as pd
import csv
from termcolor import colored

from HCGB import functions
import HCGB.functions.aesthetics_functions as HCGB_aes

####################
def generate_DE(dataframe_results, Debug, outfolder, type_analysis='miRNA'):
	"""Builds final expression matrices comparing all samples.
	
		Generates three .csv for each software used:
		miRNA_expression-soft_name_dup.csv: counts with duplicated reads for each sample
		miRNA_expression-soft_name.csv: final matrix, counts of each isomiR (with miRNA and variant info) without duplicated reads 
		miRNA_expression-soft_name_seq.csv: table with the miRTop identifier and the corresponding DNA sequence
		
		miRNA is the default analysis but other can be provided such as tRNA, piRNA, etc
		
		:param dataframe_results: dataframe with the paths of the outputs of each sample and software
		:param Debug: display complete log
		:param outfolder: output folder
		
	    :returns: None
	"""
	
	## get results dictionary for each software employed 
	soft_list = dataframe_results.soft.unique()
	
	dataframe_results = dataframe_results.reset_index()
	
	## debugging messages
	if Debug:
		print ("## Debug:")
		print ("soft_list")
		print (soft_list)

	## generate results
	for soft_name in soft_list:
		
		print ("\n+ Summarizing results for software: ", soft_name)
		
		# retrieve files in dictionary
		tmp_df = dataframe_results[ dataframe_results['soft'] == soft_name].set_index('name')
		dict_files = tmp_df['filename'].to_dict()

		if Debug:
			print ("## Debug:")
			print ("tmp_df")
			print (tmp_df)
			print ("dict_files")
			print (dict_files)

		## get data and discard duplicate UIDs if any
		(all_data, all_seqs) = generate_matrix(dict_files, soft_name.lower(), Debug, type_analysis=type_analysis)
		all_data_filtered, all_data_duplicated = discard_UID_duplicated(all_data, type_res=type_analysis)
			
		
		## dump data in folder provided
		csv_outfile = os.path.join(outfolder, type_analysis + '_expression-' + soft_name)
		all_data_filtered.to_csv(csv_outfile + ".csv", quoting=csv.QUOTE_NONNUMERIC)
		all_data_duplicated.to_csv(csv_outfile + '_dup.csv', quoting=csv.QUOTE_NONNUMERIC)
		all_seqs.to_csv(csv_outfile + '_seq.csv', quoting=csv.QUOTE_NONNUMERIC)

####################
def discard_UID_duplicated(df_data, type_res="miRNA"):
	"""
	"""
	
	## get data index
	df_data['ID'] = df_data.index
	new_data = df_data.filter(['ID'], axis=1)	

	# split ID (hsa-let-7a-2-3p&NA&qNkjr6Ov2) into miRNA, variant and UID
	tmp = new_data['ID'].str.split('&', expand = True)
	
	new_data[type_res]  = tmp[0]
	new_data['variant']  = tmp[1]
	new_data['UID']  = tmp[2]
	
	## count 
	count_groups = new_data.groupby('UID').count()
	## print to file?
	
	## get duplicated
	bigger1count = count_groups[ count_groups['ID'] > 1 ]
	## print counts to file?
	
	## get list of UIDs duplicate
	bigger1count_list = bigger1count.index.to_list()
	duplicates = new_data[new_data['UID'].isin(bigger1count_list)]
	## print duplicate to file?

	## get duplicated data
	duplicates_indes_list = duplicates.index.to_list()
	duplicates_expression = df_data[df_data.index.isin(duplicates_indes_list)]
	#duplicates_expression['UID'] = duplicates_expression['ID'].str.split('&', expand = True)[2]
	duplicates_expression = duplicates_expression.drop(['ID'], axis=1)
	duplicates_expression.index.name = "ID"
	
	## get clean data
	clean_data_expression = df_data[~df_data.index.isin(duplicates_indes_list)]
	#clean_data_expression['UID'] = clean_data_expression['ID'].str.split('&', expand = True)[2]
	clean_data_expression =	clean_data_expression.drop(['ID'], axis=1)
	clean_data_expression.index.name = "ID"
	## Fix this error
	## A value is trying to be set on a copy of a slice from a DataFrame.
	## Try using .loc[row_indexer,col_indexer] = value instead
	## 
	## See the caveats in the documentation: http://pandas.pydata.org/pandas-docs/stable/indexing.html#indexing-view-versus-copy
	##   clean_data_expression['UID'] = clean_data_expression['ID'].str.split('&', expand = True)[2]
	
	return (clean_data_expression, duplicates_expression)

####################
def generate_matrix(dict_files, soft_name, Debug, type_analysis="miRNA"):
	"""
	"""
	######################################################
	### For a list of files, generates a count matrix with a unique index id by merging:
	### information provided within each file: name, variant and UID by '&'
	### e.g. AlaAGC&3'-tRF&tRF-16-KSP185D
	### e.g. hsa-let-7a-2-3p&NA&qNkjr6Ov2
	### 
	### It returns a dataframe containing for each index generated
	### count values for each sample (sample_name) in columns
	###
	#########################################################
	all_data = pd.DataFrame()
	seq_all_data = pd.DataFrame()
	for sample, this_file in dict_files.items():
		
		new_data=pd.DataFrame()
		
		print ('+ Reading information from sample: ', sample)	
		
		## 
		if functions.files_functions.is_non_zero_file(this_file):
			data = pd.read_csv(this_file, sep='\t')
		else:
			print ('\t - Information not available for sample: ', sample)	
			continue
		##
		if (data.size == 0):
			print ('\t - Information not available for sample: ', sample)  
			continue

		####
		if type_analysis=="miRNA":
			
			## ------------------------------------------ ##
			## Create matrix for miRNA results
			## ------------------------------------------ ##
			
			## get info, generate unique name and merge for samples
			## header of tsv files: 
			## UID	Read	miRNA	Variant	iso_5p	iso_3p	iso_add3p	iso_snp	sRNAbench
	
			data['Variant'].fillna('NA', inplace=True)
			data['unique_id'] = data.apply(lambda data: data['miRNA'] + '&' + data['Variant'] + '&' + data['UID'], axis=1)
	
			## parse according to software
			if (soft_name == 'srnabench'):
				## sRNAbench mirtop creates a column id with sRNAbench instead of sample name
				new_data = data.filter(['unique_id', 'sRNAbench'], axis=1)
				new_data = new_data.set_index('unique_id')
				new_data = new_data.rename(columns={'sRNAbench': sample})
	
			if (soft_name == 'optimir'):
				## OptimiR mirtop creates a column containing sample name and other tags (trim, joined, fastq...)
				regex=re.compile(sample + '.*')
				search_list = list(filter(regex.match, data.columns.values.tolist()))
				new_data = data.filter(['unique_id', search_list[0]], axis=1)
				new_data = new_data.set_index('unique_id')
				new_data = new_data.rename(columns={search_list[0]: sample})
	
			if (soft_name == 'miraligner'):
				## miraligner mirtop creates a column containing sample name and other tags (trim, joined, fastq...)
				new_data = data.filter(['unique_id', sample], axis=1)
				new_data = new_data.set_index('unique_id')
		
		####
		elif "tRF" in type_analysis: ## tRF-amb; tRF-exc, tRF

			if Debug:
				HCGB_aes.debug_message(type_analysis + " analysis: ", color)

			## ------------------------------------------ ##
			## Create matrix for tRNA results
			## ------------------------------------------ ##
			## UID	Read	tRNA	variant	ident	expression	soft\n'
			
			data['variant'].fillna('NA', inplace=True)
			
			## parse according to software
			if (soft_name == 'mintmap'):
				data['unique_id'] = data.apply(lambda data: data['tRNA'] + '&' + data['variant'] + '&' + data['UID'], axis=1)
				new_data = data.filter(['unique_id', 'expression'], axis=1)
				new_data = new_data.set_index('unique_id')
				new_data = new_data.rename(columns={'expression': sample})
			
			## TODO
			#else:
			#	data['unique_id'] = data.apply(lambda data: data['tRNA'] + '&' + data['variant'] + '&' + data['UID'], axis=1)
			#	new_data = data.filter(['unique_id', 'XXXX'], axis=1) ## TODO
			#	new_data = new_data.set_index('unique_id')			  ## TODO
			#	new_data = new_data.rename(columns={'XXXX': sample})  ## TODO
				
		else:
			print()
			## add new
		
		## sequence information
		seq_data = data.filter(['UID', 'Read'], axis=1)	
		seq_data = seq_data.set_index('UID')
		seq_all_data = seq_all_data.append(seq_data, sort=True).drop_duplicates('Read')

		## debugging messages
		if Debug:
			print ("*** DEBUG: data for sample ***")
			print (new_data)
		
		all_data = pd.concat([all_data, new_data], axis=1, sort=True)

	##
	## debugging messages
	if Debug:
		print ("*** DEBUG: data for all samples ***")
		print (all_data)
		print ("*** DEBUG: data for sequences all samples ***")
		print (seq_all_data)
	
	return (all_data, seq_all_data)	

######

######
def main():
    ## this code runs when call as a single script

    ## control if options provided or help
    if len(sys.argv) > 1:
        print ("")
    else:
        print ("Provide file containing sample,file")
        exit()        

    dictionary_info = functions.main_functions.file2dictionary(sys.argv[1], ",")
    print (dictionary_info)

    ## get data
    (all_data, all_seqs) = generate_matrix(dictionary_info, "miraligner", False)

    ## discard duplicate UIDs if any
    all_data_filtered, all_data_duplicated = discard_UID_duplicated(all_data)

    ## dump data in folder provided
    outfolder = "./"
    csv_outfile = os.path.join(outfolder, 'miRNA_expression')
    all_data_filtered.to_csv(csv_outfile + ".csv", quoting=csv.QUOTE_NONNUMERIC)
    all_data_duplicated.to_csv(csv_outfile + '_dup.csv', quoting=csv.QUOTE_NONNUMERIC)
    all_seqs.to_csv(csv_outfile + '_seq.csv', quoting=csv.QUOTE_NONNUMERIC)



        
######
if __name__== "__main__":
    main()
    




#!/usr/bin/env python3
############################################################
## Jose F. Sanchez, Marta Lopez & Lauro Sumoy             ##
## Copyright (C) 2019-2021 Lauro Sumoy Lab, IGTP, Spain   ##
############################################################
"""
This module performs tRNA analysis with several possible tools: 
MINTMap, ... 

It shows results unifiying formats by using the miRTop nomenclature.
"""
## import useful modules
import os
import sys
import re
import time
from io import open
import shutil
import concurrent.futures
import pandas as pd
from termcolor import colored

## import my modules
from HCGB import sampleParser
from HCGB import functions
from XICRA.config import set_config
from XICRA.modules import help_XICRA
from XICRA.scripts import generate_DE
from XICRA.scripts import MINTMap_caller

##############################################
def run_tRNA(options):
    '''
    '''
    
        ## init time
    start_time_total = time.time()

    ##################################
    ### show help messages if desired    
    ##################################
    if (options.help_format):
        ## help_format option
        help_XICRA.help_fastq_format()
    elif (options.help_project):
        ## information for project
        help_XICRA.project_help()
        exit()
    elif (options.help_tRNA):
        ## information for join reads
        help_XICRA.help_tRNA()
        exit()
        
    ## debugging messages
    global Debug
    if (options.debug):
        Debug = True
    else:
        Debug = False
        
    ### set as default paired_end mode
    if (options.single_end):
        options.pair = False
    else:
        options.pair = True
    
    functions.aesthetics_functions.pipeline_header('XICRA')
    functions.aesthetics_functions.boxymcboxface("tRNA analysis")
    print ("--------- Starting Process ---------")
    functions.time_functions.print_time()

    ## absolute path for in & out
    input_dir = os.path.abspath(options.input)
    outdir=""

    ## set mode: project/detached
    if (options.detached):
        outdir = os.path.abspath(options.output_folder)
        options.project = False
    else:
        options.project = True
        outdir = input_dir        
    
    ## user software selection
    print ("+ Software for tRNA analysis selected:")
    print (options.soft_name)
    
    ## get files
    print ('+ Getting files from input folder... ')
    if options.pair:
        options.pair = False ## set paired-end to false for further prepocessing
        if options.noTrim:
            print ('+ Mode: fastq.\n+ Extension: ')
            print ("[ fastq, fq, fastq.gz, fq.gz ]\n")
            pd_samples_retrieved = sampleParser.files.get_files(options, input_dir, "fastq", ("fastq", "fq", "fastq.gz", "fq.gz"), options.debug)
        else:
            print ('+ Mode: join.\n+ Extension: ')
            print ("[_joined.fastq]\n")
            pd_samples_retrieved = sampleParser.files.get_files(options, input_dir, "join", ['_joined.fastq'], options.debug)
    else:
        if options.noTrim:
            print ('+ Mode: fastq.\n+ Extension: ')
            print ("[ fastq, fq, fastq.gz, fq.gz ]\n")
            pd_samples_retrieved = sampleParser.files.get_files(options, input_dir, "fastq", ("fastq", "fq", "fastq.gz", "fq.gz"), options.debug)
        else:
            print ('+ Mode: join.\n+ Extension: ')
            print ("[_joined.fastq]\n")
            pd_samples_retrieved = sampleParser.files.get_files(options, input_dir, "trim", ['_trim'], options.debug)
    
    ## debug message
    if (Debug):
        print (colored("**DEBUG: pd_samples_retrieve **", 'yellow'))
        print (pd_samples_retrieved)

    ## species
    print ("+ Species provided:", options.species)

    ## set database path if necessary
    if not (options.database):
        install_path =  os.path.dirname(os.path.realpath(__file__))
        options.database = os.path.join(install_path, "db_files") 
    else:
        options.database = os.path.abspath(options.database)
    
    ## generate output folder, if necessary
    if not options.project:
        print ("\n+ Create output folder(s):")
        functions.files_functions.create_folder(outdir)
    
    ## for samples
    outdir_dict = functions.files_functions.outdir_project(outdir, options.project, pd_samples_retrieved, "tRNA", options.debug)
    
    ## optimize threads
    name_list = set(pd_samples_retrieved["new_name"].tolist())
    threads_job = functions.main_functions.optimize_threads(options.threads, len(name_list)) ## threads optimization
    max_workers_int = int(options.threads/threads_job)

    ## to FIX: MINTmap requires to chdir to folder to create results
    max_workers_int = 1

    ## debug message
    if (Debug):
        print (colored("**DEBUG: options.threads " +  str(options.threads) + " **", 'yellow'))
        print (colored("**DEBUG: max_workers " +  str(max_workers_int) + " **", 'yellow'))
        print (colored("**DEBUG: cpu_here " +  str(threads_job) + " **", 'yellow'))

    print ("+ Create a tRNA analysis for each sample retrieved...")    
    
    ## call tRNA_analysis: 
    ## Get user software selection: mintmap, ...
    
    ## dictionary results
    global results_df
    results_df = pd.DataFrame(columns=("name", "soft", "type", "filename"))
    
    # Group dataframe by sample name
    sample_frame = pd_samples_retrieved.groupby(["new_name"])
    
    ## send for each sample
    with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers_int) as executor:
        commandsSent = { executor.submit(tRNA_analysis, 
                                         sorted(cluster["sample"].tolist()), 
                                         outdir_dict[name], name, threads_job, 
                                         options.soft_name, options.species, 
                                         options.database, Debug): name for name, cluster in sample_frame }

        for cmd2 in concurrent.futures.as_completed(commandsSent):
            details = commandsSent[cmd2]
            try:
                data = cmd2.result()
            except Exception as exc:
                print ('***ERROR:')
                print (cmd2)
                print('%r generated an exception: %s' % (details, exc))

    print ("\n\n+ tRNA analysis is finished...")
    print ("+ Let's summarize all results...")
    
    ## outdir
    outdir_report = functions.files_functions.create_subfolder("report", outdir)
    expression_folder = functions.files_functions.create_subfolder("tRNA", outdir_report)

    ## debugging messages
    if options.debug:
        print (results_df)
        
    ## merge all parse gtf files created
    print ("+ Summarize tRNA analysis for all samples...")
    
    if 'mintmap' in options.soft_name:
        results_df = results_df.set_index('type')
        
        ## exclusive tRFs
        print ("\n\n+ Parsing exclusive tRNA analysis for all samples...")
        generate_DE.generate_DE(results_df.filter(like="amb", axis=0).set_index('name'), 
                                options.debug, expression_folder,  type_analysis="tRF-amb")
        
        ## amb tRFs
        print ("\n\n+ Parsing ambiguous tRNA analysis for all samples...")
        generate_DE.generate_DE(results_df.filter(like="exc", axis=0).set_index('name'), 
                                options.debug, expression_folder,  type_analysis="tRF-exc")
    else:
        generate_DE.generate_DE(results_df, options.debug, expression_folder,  type_analysis="tRNA")

    print ("\n*************** Finish *******************")
    start_time_partial = functions.time_functions.timestamp(start_time_total)
    print ("\n+ Exiting tRNA module.")
    return()


#########################################
def tRNA_analysis(reads, folder, name, threads, soft_list, species, database, Debug):
    
    ##
    for soft in soft_list:
        if (soft == "mintmap"):
            ## create mintmap
            MINTmap_folder = functions.files_functions.create_subfolder('mintmap', folder)
            code_success = MINTMap_caller.MINTmap_caller(MINTmap_folder, reads, name, threads, species, Debug)   
            
            if not code_success:
                print ('** Some error ocurred during MINTmap analysis for sample %s...' %name)
                return ()
            
            ## save results in dataframe
            filename_amb = os.path.join(MINTmap_folder, 'mintmap_parse', name + '_amb.tsv')
            filename_exc = os.path.join(MINTmap_folder, 'mintmap_parse', name + '_exc.tsv')
            
            results_df.loc[len(results_df)] = name, soft, "amb", filename_amb
            results_df.loc[len(results_df)] = name, soft, "exc", filename_exc
    